#include "coupledwalk.ih"

CoupledWalk::CoupledWalk(string const &id, string const &playerClass)
  : Behavior("CoupledWalk", id, playerClass)
{
  // double amplitude, double period, double phaseOffset, double offset, double coupling
  /*
        &lleg2;: 4period 0 60, -4period 1.570796327 0;
        &rleg2;: -4period 0 60, 4period 1.570796327 0;

        &lleg4;: -8period 0 -50;
        &rleg4;: 8period 0 -50;

        &lleg5;: 4period 0 18, 4period 1.570796327 0;
        &rleg5;: -4period 0 18, -4period 1.570796327 0;
  */
  
  double coupling = 3;
  double period = 1.0;
  
  // Create sines
  d_sines[Types::LLEG2] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::LLEG2].push_back(new CoupledSineFunction(Math::degToRad(4),period, 0, Math::degToRad(60.0), coupling));
  d_sines[Types::LLEG2].push_back(new CoupledSineFunction(Math::degToRad(-4),period, M_PI / 2.0, 0.0, coupling));
  
  d_sines[Types::LLEG3] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::LLEG3].push_back(new CoupledSineFunction(Math::degToRad(7),period, 0, 0, coupling));

  d_sines[Types::LLEG4] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::LLEG4].push_back(new CoupledSineFunction(Math::degToRad(-8),period, 0, Math::degToRad(-50.0), coupling));
  
  d_sines[Types::LLEG5] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::LLEG5].push_back(new CoupledSineFunction(Math::degToRad(4),period, 0, Math::degToRad(18.0), coupling));
  d_sines[Types::LLEG5].push_back(new CoupledSineFunction(Math::degToRad(4),period, M_PI / 2, 0.0, coupling));

  d_sines[Types::LLEG6] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::LLEG6].push_back(new CoupledSineFunction(Math::degToRad(-7),period, 0, 0, coupling));
  
  
  d_sines[Types::RLEG2] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::RLEG2].push_back(new CoupledSineFunction(Math::degToRad(-4),period, 0, Math::degToRad(60.0), coupling));
  d_sines[Types::RLEG2].push_back(new CoupledSineFunction(Math::degToRad(4),period, M_PI / 2.0, 0.0, coupling));

  d_sines[Types::RLEG3] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::RLEG3].push_back(new CoupledSineFunction(Math::degToRad(7),period, 0, 0, coupling));

  d_sines[Types::RLEG4] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::RLEG4].push_back(new CoupledSineFunction(Math::degToRad(8),period, 0, Math::degToRad(-50.0), coupling));

  d_sines[Types::RLEG5] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::RLEG5].push_back(new CoupledSineFunction(Math::degToRad(-4),period, 0, Math::degToRad(18.0), coupling));
  d_sines[Types::RLEG5].push_back(new CoupledSineFunction(Math::degToRad(-4),period, M_PI / 2, 0.0, coupling));

  d_sines[Types::RLEG6] = vector<rf<CoupledSineFunction> >();
  d_sines[Types::RLEG6].push_back(new CoupledSineFunction(Math::degToRad(-7),period, 0, 0, coupling));
  
  d_tree = new AST::Node(sequenceType);
  d_tree->addChild(new AST::Node(andType));
  for (unsigned i = 0; i < Types::NJOINTS; ++i)
    d_tree->getChild(0)->addChild(new AST::Node(orType));
}

