#include "cubicspline.ih"

double CubicSpline::calcAngle(unsigned slot)
{
	// See if there is actually a spline defined for this slot
	map<unsigned, Definition>::iterator iter = d_joints.find(slot);
	
	// Return if there is no such joint definition
	if (iter == d_joints.end())
		throw NoGoalException();

	Definition &joint = iter->second;
	
	if (!joint.periodic.active())
		throw NoGoalException();

	double ampfactor = joint.periodic.amplitude();
	double position = joint.periodic.sampleTime();
	
	// Calculate the angle of the joint
	double angle = ampfactor * splineinterpolation(joint.coefficients, position);
	
	// Convert from deg to rad
	return angle * (M_PI / 180.0);
}

