#include "cubicspline.ih"

void
CubicSpline::closeLoop(vector<pair<double, double> > &points) const
{
	// Let's check the starting and ending points
	if (points.front().first != 0 && points.back().first != 1.0)
	{
		// Interpolate the first and last point
		double y = (points.front().second + points.back().second) / 2.0;
		
		// Add first point at t = 0
		points.insert(points.begin(), pair<double, double>(0.0, y));
		
		// Add last point at t = joint.period
		points.push_back(pair<double, double>(1.0, y));
	}
	else if (points.front().first != 0.0)
	{
		// Add first point, with same value as last point
		points.insert(points.begin(), pair<double, double>(0.0, points.back().second));
	}
	else if (points.back().first != 1.0)
	{
		// Add last point, with same value as first point
		points.push_back(pair<double, double>(1.0, points.front().second));
	}
	else if (points.front().second != points.back().second)
	{
		// Set value of last point to the same value as the first to
		// ensure periodicity
		points.back().second = points.front().second;
	}
}

