#ifndef _BATS_CUBIC_SPLINE_H_
#define _BATS_CUBIC_SPLINE_H_

#include "behavior.hh"
#include "periodicjoint.hh"
#include "interpolation/spline3.h"
#include "periodicinterface.hh"
#include <vector>
#include <exception>

namespace bats
{
	class CubicSpline : public Behavior, public PeriodicInterface
	{
		private:
			class NoGoalException : public std::exception
			{
			};

			struct Definition
			{
				PeriodicJoint periodic;
				ap::real_1d_array coefficients;
			};
			
			std::map<unsigned, Definition> d_joints;

			double calcAngle(unsigned slot);
			
			void parseJoint(XMLNode node);
			void addPoint(std::vector<std::pair<double, double> > &points, double t, double y) const;
			void closeLoop(std::vector<std::pair<double, double> > &points) const;
			void smoothTransition(std::vector<std::pair<double, double> > &points) const;
			void phaseShift(std::vector<std::pair<double, double> > &points, double phase) const;
			
			virtual rf<Goal> generateGoal(unsigned step, unsigned slot);
			virtual rf<State> getCurrentState();
			virtual ConfidenceInterval getCapability(rf<State> s, rf<Goal> g) { return ConfidenceInterval(0.5, 0.2); }

		public:
			CubicSpline(std::string const &id, std::string const &playerClass);
			
			double startPosition(Types::Joint joint);
			void reset();
	};
};

#endif /* _BATS_CUBIC_SPLINE_H_ */

