#include "cubicspline.ih"

void
CubicSpline::smoothTransition(vector<pair<double, double> > &points) const
{
	// To ensure a smooth transition, add first two elements to the back of the vector
	// and the last two elements to the front (at the correct t of course). This
	// function assumes that the points have been normalized from 0 - T
	
	// Get second point
	pair<double, double> second = *(points.begin() + 1);
	
	// Get before last point
	pair<double, double> beforelast = *(points.end() - 2);
	
	// Add this 'second' point to the end, with corrected t
	second.first += 1.0;
	points.push_back(second);
	
	// Now add the 'beforelast' point in the front, with corrected t
	beforelast.first -= 1.0;
	points.insert(points.begin(), beforelast);
}

