#include "cubicspline.ih"

#include <typeinfo>

double
CubicSpline::startPosition(Types::Joint joint)
{
	// Find the start position of this joint, to do this, we need to find the actual behavior first
	if (d_tree->size() == 0)
		return 0;
	
	// Get the first step
	rf<AST::Node> first = d_tree->getChild(0);

	// Iterate over all the slots in the step
	for (size_t idx = 0; idx < first->size(); ++idx)
	{
		rf<AST::Node> child = first->getChild(idx);

		if (!child->hasChildren())
			continue;

		rf<AST::Node> slot = child->getChild(0);
		
		if (slot->getType() != behaviorType)
			continue;

		try
		{
			rf<BehaviorNode> behavior = rf_cast<BehaviorNode>(slot);
			JointInterface &ch = dynamic_cast<JointInterface &>(*(behavior->getBehavior()));
			
			if (ch.joint() == joint)
			{
				// We found our joint at the slot 'idx'
				map<unsigned, Definition>::iterator iter = d_joints.find(idx);
				
				// Check if the actual joint has parameters
				if (iter == d_joints.end())
					continue;
				
				Definition &def = iter->second;
				double angle = splineinterpolation(def.coefficients, 0);
				
				if (def.periodic.modulationSpan() != 0 && def.periodic.amplitudeModulation())
					angle *= def.periodic.amplitudeStart();
				
				return angle * (M_PI / 180.0);
			}
		}
		catch (bad_cast)
		{
		}
	}
	
	return 0.0;
}

