#include "dribble.ih"
#include "math.hh"

rf<Behavior::Goal> Dribble::generateGoal(unsigned step, unsigned slot)
{
  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();
  
  // con->addVar("Var", 0, 1);

	WorldModel &wm = SWorldModel::getInstance();
	Vector3D pos,dir;
  Vector3D oppGoal, goalPos;
  Vector3D ballPos = wm.getObject(Types::BALL).position.getMu();
   
	Vector3D goal1Pos = wm.getObject(Types::GOAL1THEM).position.getMu();
	Vector3D goal2Pos = wm.getObject(Types::GOAL2THEM).position.getMu();
	goal1Pos = (goal1Pos * 7 + goal2Pos) / 8.0;
	goal2Pos = (goal2Pos * 7 + goal1Pos) / 8.0;
	
	
	ballPos.setZ(0);
	goal1Pos.setZ(0);
	goal2Pos.setZ(0);

	//TWEAK: Adjust ballpos slighty more to agent so agent does not walk by ball.
//	ballPos = ballPos - ballPos.normalize() * 0.1;

	Vector3D ballToGoal1 = goal1Pos - ballPos;
	Vector3D ballToGoal2 = goal2Pos - ballPos;
	
	
	double ballAngle = atan2(ballPos[0], ballPos[1]) / M_PI * 2;
	double goal1Angle = atan2(goal1Pos[0], goal1Pos[1]) / M_PI * 2;
	double goal2Angle = atan2(goal2Pos[0], goal2Pos[1]) / M_PI * 2;
  
	
  if (ballAngle > goal1Angle && ballAngle < goal2Angle)
	{
  	goalPos = ballPos; 
		_debugLevel1("Walkstraight angleToBall " << ballAngle << " goal1angle: " << goal1Angle);
		
		pos = ballPos;
  	dir = ballPos;
  }
	else 
	{		
		double ballGoal1Angle	= Math::atSameSideOf(ballPos, goal1Pos, Vector3D(0,0,0)) 
													? M_PI - ballPos.angle(ballToGoal1) 
													: ballPos.angle(ballToGoal1);
		double ballGoal2Angle	= Math::atSameSideOf(ballPos, goal2Pos, Vector3D(0,0,0)) 
													? M_PI - ballPos.angle(ballToGoal2) 
													: ballPos.angle(ballToGoal2);
												
  		_debugLevel4("goal1: " << ballGoal1Angle * 360 / (2*M_PI) << "    goal2: " << ballGoal2Angle * 360 / (2*M_PI));
  											
													
		oppGoal = (fabs(ballGoal1Angle) < fabs(ballGoal2Angle))
						?	goal1Pos
						:	goal2Pos;
		
		_debugLevel4("same? " << Math::atSameSideOf(ballPos, goal1Pos, Vector3D(0,0,0)) << " goal1Pos is goal: " << (oppGoal == goal1Pos));

  		Vector3D tweak = Math::calcPerpend(oppGoal - ballPos) * d_tweakPos;
  		if((tweak+ballPos).length() > (tweak * -1.0+ballPos).length())
  		  tweak = tweak * -1.0;

       pos = ballPos+tweak;
       dir = oppGoal;		 
        
  	} 
 
    WorldModel::player_iterator it = wm.beginOpponents();
    WorldModel::player_iterator end = wm.endOpponents();

    double closest_to_ball = 999999.0;
    for (; it != end; ++it)
    {
        double dist = (it->second.getPosition() - wm.getObject(Types::BALL).position.getMu()).length();
        if (dist < closest_to_ball)
            closest_to_ball = dist;
    }

  //correctIfCollides(Vector3D const &target, Vector3D goal, size_t depth, bool surpressBall, bool onlyPlayers)
   if (closest_to_ball < d_avoid_players_threshold)
        pos = Avoider::correctIfCollides(pos, Vector3D(0, 0, 0), 1, PLAYERS);
    else
        pos = Avoider::correctIfCollides(pos, Vector3D(0, 0, 0), 1, PLAYERS | ENEMIES);
   

	Vector3D subPos;
	if (pos.length() * d_curveFactor > d_curveConst)
	{
		 subPos = pos - dir.normalize() * pos.length()*d_curveFactor;		 
		 _debugLevel1("dribble factor: " << (dir.normalize() * pos.length()*d_curveFactor).length() << endl);
	}
	else
	{
		 subPos = pos - dir.normalize() * d_curveConst + dir.normalize() * 2 * (d_curveConst - pos.length() * d_curveFactor);		 
		_debugLevel1("dribble const: " << (dir.normalize() * d_curveConst + dir.normalize() * 2 * (d_curveConst - pos.length() * d_curveFactor)).length() << endl);
	}
	 
	d_currentDribbleAngle	= Math::atSameSideOf(ballPos, dir, Vector3D(0,0,0)) 
													? fabs(M_PI - ballPos.angle(dir) )
													: fabs(ballPos.angle(dir)); 
													
	con->addVar("Pos", subPos);
	con->addVar("FinalPos", pos);
	con->addVar("Dir", dir);
	con->addVar("TargetSpeed", d_targetSpeed, d_targetSpeed);
  con->addVar("BrakeDistance", d_brakeDistance, d_brakeDistance);
  
  //	con->addVar("SlowOnAngle", 8, 8);
    
  return goal;
}
