#include "goodposition.ih"

rf<Behavior::Goal> GoodPosition::generateGoal(unsigned step, unsigned slot)
{
  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();
  
  WorldModel& wm = SWorldModel::getInstance();
  
  // Find position to go to
  Vector3D pos;
  
  if (d_goal->findDeep("PosX"))
    pos = d_goal->getVector("Pos");
  else
    pos = wm.getObject(Types::BALL).position.getMu();
  
  pos[2] = 0;
  
  double dist = pos.length();
  double brakeFactor = dist < d_brake_distance ? 1.0 - dist / d_brake_distance : 0.0;
  _debugLevel1("Brake: " << dist << " " << d_brake_distance << " " << brakeFactor);
  
  double max = fabs(pos[0]) > fabs(pos[1]) ? fabs(pos[0]) : fabs(pos[1]);
  Vector3D dir = pos / max * (1.0 - brakeFactor);
  
  _debugLevel1("Pos: " << pos << ", Dir: " << dir);
  
  // Determine factors for each joint
  for (unsigned i = 0; i < Types::NJOINTS; ++i)
  {
    double f = 1.0;

    // Modulate right 
    if (i == Types::LLEG3 || i == Types::LLEG6 ||
        i == Types::RLEG3 || i == Types::RLEG6)
      f = dir[0];

    ostringstream jout;
    jout << "AmpFactor" << i << "-" << 0;
    con->addVar(jout.str(), f, f);

    // Modulate forward 
    if (i == Types::LLEG2 || i == Types::LLEG5 ||
        i == Types::RLEG2 || i == Types::RLEG5)
    {
      f = dir[1];
      ostringstream jout2;
      jout2 << "AmpFactor" << i << "-" << 1;
      con->addVar(jout2.str(), f, f);
    }
    

  }

  return goal;
}
