#include "goodpositionangle.ih"

rf<Behavior::Goal> GoodPositionAngle::generateGoal(unsigned step, unsigned slot)
{
  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();
  
  WorldModel& wm = SWorldModel::getInstance();
  
  if (slot == 0)
  {
    // Find position to go to
    Vector3D pos;
    double angle = 0;
    
    
    if (d_goal->findDeep("PosX"))
      pos = d_goal->getVector("Pos");
    else
      pos = wm.getObject(d_defaultPos).position.getMu();

    //Change the position if the ball is in the way and behind us
    //  so if we keep walking backwards, we will avoid the ball by
    //  having freePart translate the position to a new safe position next
    //  to the ball
    avoidBallBehind(&pos);

    //correctIfCollides(Vector3D const &target, Vector3D goal, size_t depth, bool surpressBall, bool onlyPlayers)
    pos = Avoider::correctIfCollides(pos, Vector3D(0, 0, 0), 1, PLAYERS);
    /*
    //Only used by walkToAngle
    if (d_goal->findDeep("FinalPosX"))
    	pos = d_goal->getVector("FinalPos");*/
    
    pos[2] = 0;
    
    if (d_goal->findDeep("Angle"))
      angle = d_goal->getMean("Angle");
    else if (d_goal->findDeep("DirX"))
    {
      Vector3D d = d_goal->getVector("Dir");
      _debugLevel4("d: " << d);
      angle = atan2(d[0], d[1]);
    }
    else
    {
      //Otherwise just point at the position
      angle = Math::cartesianToPolar(pos).getY();
      
    }

    _debugLevel4("Angle: " << angle);
    
    double turnDir = angle < 0.0 ? -1 : 1;
    
    double dist = pos.length();
    double brakeFactor = dist < d_brake_distance ? 1.0 - dist / d_brake_distance : 0.0;
    _debugLevel4("Brake: " << dist << " " << d_brake_distance << " " << brakeFactor);
    
    double max = fabs(pos[0]) > fabs(pos[1]) ? fabs(pos[0]) : fabs(pos[1]);
    Vector3D dir = pos / max * (1.0 - brakeFactor);
    
    _debugLevel4("Pos: " << pos << ", Dir: " << dir);
    
    // Determine factors for each joint
    for (unsigned i = 0; i < Types::NJOINTS; ++i)
    {
      double f = 1.0;

      // Modulate right 
      if (i == Types::LLEG3 || i == Types::LLEG6 ||
          i == Types::RLEG3 || i == Types::RLEG6)
        f = dir[0];
      // Modulate turn
      else if (i == Types::LLEG1 || i == Types::RLEG1)
        f = fabs(angle) < d_min_angle ? turnDir * (fabs(angle) / d_min_angle) : turnDir;

      ostringstream jout;
      jout << "AmpFactor" << i << "-" << 0;
      con->addVar(jout.str(), f, f);

      // Modulate forward 
      if (i == Types::LLEG2 || i == Types::LLEG5 ||
          i == Types::RLEG2 || i == Types::RLEG5)
      {
        f = dir[1];
        ostringstream jout2;
        jout2 << "AmpFactor" << i << "-" << 1;
        con->addVar(jout2.str(), f, f);
      } 
    }
  }
  // Doesnt work
  else
  {
    AgentModel& am = SAgentModel::getInstance();
    Vector3D gyro = am.getGyro();
    _debugLevel1(wm.getTime() << " Gyro: " << gyro);
    double speed = 0.001 * gyro[0];
    con->addVar("Speed", speed, speed);
  }
  return goal;
}
