/*
 *  Little Green BATS (2007), AI department, University of Groningen
 *
 *  Authors: 	Martin Klomp (martin@ai.rug.nl)
 *		Mart van de Sanden (vdsanden@ai.rug.nl)
 *		Sander van Dijk (sgdijk@ai.rug.nl)
 *		A. Bram Neijt (bneijt@gmail.com)
 *		Matthijs Platje (mplatje@gmail.com)
 *
 *	All students of AI at the University of Groningen
 *  at the time of writing. See: http://www.ai.rug.nl/
 *
 *  Date: 	July 27, 2007
 *
 *  Website:	http://www.littlegreenbats.nl
 *
 *  Comment:	Please feel free to contact us if you have any 
 *		problems or questions about the code.
 *
 *
 *  License: 	This program is free software; you can redistribute 
 *		it and/or modify it under the terms of the GNU General
 *		Public License as published by the Free Software 
 *		Foundation; either version 3 of the License, or (at 
 *		your option) any later version.
 *
 *   		This program is distributed in the hope that it will
 *		be useful, but WITHOUT ANY WARRANTY; without even the
 *		implied warranty of MERCHANTABILITY or FITNESS FOR A
 *		PARTICULAR PURPOSE.  See the GNU General Public
 *		License for more details.
 *
 *   		You should have received a copy of the GNU General
 *		Public License along with this program; if not, write
 *		to the Free Software Foundation, Inc., 59 Temple Place - 
 *		Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef _BATS_GOODPOSITIONANGLE_HH_
#define _BATS_GOODPOSITIONANGLE_HH_

#include "behavior.hh"

namespace bats
{

  /**
   *  Behavior: Turn to a good angle.
   */
  class GoodPositionAngle : public Behavior
  {

    enum StableCheck
    {
      ON_FEET,
      NOT_FALLING,
    };
    
    Types::Object d_defaultPos;
    double d_max_distance, d_min_distance, d_min_capable_distance;

    double d_min_angle;
    double d_brake_distance;
    bool d_avoidBehindBall;
    
    virtual rf<Goal> generateGoal(unsigned step, unsigned slot);
    virtual rf<State> getCurrentState();

    /** Get Capability
     *
     *  @param s Expected state variable: None
     *  @param g Expected goal variable: Pos
     *  @returns confidence (0.5, 0.2) when not to far away and on it feet.
     */
    virtual ConfidenceInterval getCapability(rf<State> s, rf<Goal> g);
    void avoidBallBehind(Vector3D *pos);
    StableCheck d_stableCheck;
  public:

    GoodPositionAngle(std::string const &id, std::string const &playerClass);
    
    void update();

  };
};

#endif
