#include "keeperfall.ih"

Behavior::ConfidenceInterval KeeperFall::getCapability(rf<State> s, rf<Goal> g)
{

  WorldModel& wm = SWorldModel::getInstance();
  AgentModel& am = SAgentModel::getInstance();

  // If the agent is down, or if we have kickoff.
  if(am.isDown())
    return capable(false);

  if(wm.isObjectAlive(Types::BALL))
  {
    Vector3D ballPos = wm.getObject(Types::BALL).position.getMu();
    if(ballPos.length() > d_radius)
      return capable(false);

    float angle = Math::cartesianToPolar(ballPos, true).getY();

    if(d_right)
    { //Ready to fall to the right
      if(angle > -0.5 * M_PI && angle < 0.5 * M_PI)
        return capable(true);
    }
    else //Left
    {
      if(angle > 0.5 * M_PI || angle < -0.5 * M_PI)
        return capable(true);
    }
  }
  return capable(false);
//////////OBJECT MOVEMENT WORKING, BUT TO RISKY AT THE MOMENT
  //_debugLevel4(wm.getObjectMovement(Types::BALL));
  //_debugLevel4(wm.getObjectDelay(Types::BALL));
  if(wm.isObjectAlive(Types::BALL))
  {
    //Goal must be approximately behind us
    Vector3D goalCenter = wm.getObject(Types::GOAL1US).position.getMu();
    goalCenter += wm.getObject(Types::GOAL2US).position.getMu();
    goalCenter = goalCenter / 2;
    float goalAngle = Math::cartesianToPolar(goalCenter, true).getY();
    if(goalAngle > -0.25*M_PI || goalAngle < -0.75 * M_PI)
    {
      _debugLevel3("Not in front of the goal");
      return capable(false);
    }

    //Predict position and consider
    NormalDistribution3D ballPos = wm.getObject(Types::BALL).position;
    Vector3D dBall = wm.getObjectMovement(Types::BALL);
    if(dBall.length() < 0.2 || dBall.length() > 10)
    {
      //To fast or to slow
      return capable(false);
    }

    //Ball must be moving towards us
    if(dBall.getY() > 0)
    {
      _debugLevel4("Ball moving away, yeah!");
      return capable(false);
    }
    _debugLevel4("Ball NOT moving away, oh no!");

    float v = dBall.length() / wm.getObjectDelay(Types::BALL);
    //float a = 0.01/0.026;
    float a = 0.04/0.026;
    float x = 0.5 * v * v / a; //Hoe ver hij gaat

    Vector3D zeroBallPos = ballPos.getMu();//Zero speed ball position
    _debugLevel1(dBall);
    if(dBall.length() > 0.01)
    {
      zeroBallPos += dBall.normalize() * x;
      _debugLevel2("Zero-speed ball pos: " << zeroBallPos << " actual ballpos: " << ballPos.getMu());
    }
    Vector3D closestPoint = Math::linePointClosestToPoint(ballPos.getMu(), dBall, Vector3D(0,0,0));

    //Closest intersection is to far away to reach
    if(closestPoint.length() > 1.5)
    {
      _debugLevel3("Closest length to far" << closestPoint.length());
      return capable(false);
    }

    //Closest intersection is to far away to reach
    if(closestPoint.length() < 0.2)//About half our width
    {
      _debugLevel3("Closest length to close" << closestPoint.length());
      return capable(false);
    }

    float closestAngle = Math::cartesianToPolar(closestPoint, true).getY();

    if(d_right)
    {
      if(closestAngle > 0.5 * M_PI || closestAngle < -0.5 * M_PI)
      {
        //_debugLevel3("Closest length " << closestPoint.length());
        return capable(false);
      }
    }
    else //Left
    {
      if(closestAngle < 0.5 * M_PI && closestAngle > -0.5 * M_PI)
      {
        //_debugLevel3("Closest length " << closestPoint.length());
        return capable(false);
      }
    }

    float zeroAngle = Math::cartesianToPolar(zeroBallPos, true).getY();


    if(zeroAngle < 0)//Ball will end up behind us in one second
    {
      return ConfidenceInterval(0.8, 0.2);
    }
    //_debugLevel2("zeroAngle: " << zeroAngle);
    /*
    float nextAngle = Math::cartesianToPolar(nextBallPos, true).getY();

    //We would have to fall to the wrong side
    if(nextAngle < 0 //Ball will already be behind us
       || (d_right && nextAngle > M_PI * 0.5) //Right, but we have the ball left
       || (!d_right && nextAngle < M_PI * 0.5) //Left, but we have the ball right
      )
      return capable(false);

    //The Ball must end up in our neigherhood in the second we need to fall
    if(nextBallPos.length() > d_radius+1)
      return capable(false);

    //Will the ball pass us within the radius? Then FALL (and stay down ;)
    if(Math::distanceLinePoint(ballPos.getMu(),
				 dBall,
				 Vector3D(0,0,0)) < d_radius)
    {
      _debugLevel2("Capable");

      return ConfidenceInterval(0.8, 0.2);
    }
    */

  }
  return capable(false);
}
