#include "kickball.ih"

KickBall::KickBall(string const &id, string const &playerClass)
  : Behavior("KickBall", id, playerClass),
    d_side(Types::RIGHT)
{
  d_tree = new AST::Node(sequenceType);

  // lean left/right
  d_tree->addChild(new AST::Node(andType));
  d_tree->getChild(0)->addChild(new AST::Node(orType));
  
  // position foot
  d_tree->addChild(new AST::Node(andType));
  d_tree->getChild(1)->addChild(new AST::Node(orType));
  d_tree->getChild(1)->addChild(new AST::Node(orType)); 
  
  // swing
  d_tree->addChild(new AST::Node(andType));
  for (unsigned i = 0; i < 6; ++i)
    d_tree->getChild(2)->addChild(new AST::Node(orType));
 
  XMLNodeSet params = getParams("/side");
  if(params && !params.empty())
  {
    XMLNode scriptNode = params.front();
    if (scriptNode.getContent() == "0")
      d_side = Types::LEFT;
  }
  
  d_maxBallDist = getParam("/distance/@maxball", 0.3);
  d_maxStepTime = getParam("/maxsteptime", 1.0);
  
  XMLNodeSet kickParamNodes = getParams("/kick");
  _debugLevel1(ident() << " N nodes: " << kickParamNodes.size());
  for (XMLNodeSet::iterator iter = kickParamNodes.begin(); iter != kickParamNodes.end(); ++iter)
  {
    XMLNode node = *iter;
    Vector3D dir(atof(node.getProp("dirX").c_str()),
                 atof(node.getProp("dirY").c_str()),
                 atof(node.getProp("dirZ").c_str()));
    KickParams params;
    istringstream in(node.getContent());
    string dummy;  
    in >> dummy >> params.xRange.first >> params.xRange.second
        >> dummy >> params.yRange.first >> params.yRange.second
        >> dummy >> params.maxAngle
        >> dummy >> params.footPos[0] >> params.footPos[1] >> params.footPos[2]
        >> dummy;
    params.maxAngle *= M_PI / 180.0;
    
    for (unsigned i = 0; i < 11; ++i)
      in >> params.amps[i];
    _debugLevel1(ident() << " dir: " << dir);
    d_kickParams[dir] = params;
    _debugLevel1(ident() << " N kickParams: " << d_kickParams.size());
  }
}
