#include "kickball.ih"

rf<Behavior::Goal> KickBall::generateGoal(unsigned step, unsigned slot)
{
  WorldModel& wm = SWorldModel::getInstance();
  AgentModel& am = SAgentModel::getInstance();
  
  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();
  
  Vector3D dir = Vector3D(0, 1, 0);
  if (d_goal->findDeep("DirX"))
    dir = d_goal->getVector("Dir");
  
  _debugLevel1("Dir: " << dir);
  
  Vector3D front(0,1,0);
  double dirAngle = front.angle(dir);
  if (dir[0] < 0)
    dirAngle = -dirAngle;
    
  _debugLevel1(ident() << " Angle: " << dirAngle);
  
  double minAbsAngle = M_PI;
  KickParams params;
  for (map<Vector3D, KickParams >::iterator iter = d_kickParams.begin(); iter != d_kickParams.end(); ++iter)
  {
    Vector3D d = iter->first;
    double a = front.angle(d);
    if (d[0] < 0)
      a = -a;
    if (fabs(dirAngle - a) < minAbsAngle)
    {
      params = iter->second;
      minAbsAngle = fabs(dirAngle - a);
    }
  }

  if (step == 1)
  {
    if (slot == 0)
    {
      Vector3D ballPos = wm.getObject(Types::BALL).position.getMu();
      Vector3D targetPos;
      if (d_goal->findDeep("FootPosX"))
        targetPos = ballPos + d_goal->getVector("FootPos");
      else
      {
        targetPos = params.footPos;
        targetPos = targetPos + ballPos;
        _debugLevel1("foot: " << params.footPos << ", ball: " << ballPos << ", target: " << targetPos);  
      }
      _debugLevel1("Foot pos target: " << targetPos);
      con->addVar("Target", targetPos);
    }
    else if (slot == 1)
    {
      Vector3D target = Vector3D((d_side == Types::LEFT ? -1.0 : 1.0) * 0.4 * am.getBodyPart(Types::LFOOT)->size[0],
                                     0.1 * am.getBodyPart(Types::LFOOT)->size[1],
                                     0);
      _debugLevel1("COM target: " << target);
      con->addVar("Target", target);
    }
  }
  else if (step == 2)
  {
    for (unsigned i = 0; i < 11; ++i)
    {
      ostringstream ampout;
      ampout << "Amp" << i;
      con->addVar(ampout.str(), params.amps[i], params.amps[i]);
    }
  }
  // con->addVar("Var", 0, 1);
  
  return goal;
}
