#include "kickball.ih"

Behavior::ConfidenceInterval KickBall::getCapability(rf<State> s, rf<Goal> g)
{
  AgentModel& am = SAgentModel::getInstance();
  WorldModel& wm = SWorldModel::getInstance();

  Vector3D ballPos = wm.getObject(Types::BALL).position.getMu();
  Vector3D leftFootPos = am.getBodyPart(Types::LFOOT)->transform.trans();
  Vector3D rightFootPos = am.getBodyPart(Types::RFOOT)->transform.trans();

  // Get direction to shoot at
  Vector3D dir = Vector3D(0, 1, 0);
  if (g->findDeep("DirX"))
    dir = g->getVector("Dir");
  dir[2] = 0;
   
  _debugLevel1(ident() << " Dir: " << dir);

  // We cant kick backwards!
  if (dir[1] < 0)
    return capable(false);
    
  // See what angle that is in
  Vector3D front(0,1,0);
  double dirAngle = front.angle(dir);
  if (dir[0] < 0)
    dirAngle = -dirAngle;

  _debugLevel4(ident() << " Angle: " << dirAngle);

  // Find params that kick closest to that direction
  double minAbsAngle = M_PI;
  KickParams params;
  _debugLevel4(ident() << " N kickParams: " << d_kickParams.size());
  for (map<Vector3D, KickParams >::iterator iter = d_kickParams.begin(); iter != d_kickParams.end(); ++iter)
  {
    Vector3D d = iter->first;
    double a = front.angle(d);
    if (d[0] < 0)
      a = -a;
    _debugLevel4(ident() << " d: " << d << ", a: " << a);
    if (fabs(dirAngle - a) < minAbsAngle)
    {
      params = iter->second;
      _debugLevel1("Selecting params for dir " << d);
      minAbsAngle = fabs(dirAngle - a);
    }
  }
  _debugLevel1("Min abs angle: " << minAbsAngle << " max angle: " << params.maxAngle);
  
  if (minAbsAngle > params.maxAngle)
    return capable(false);
    
  // Get foot and ball positions
  Vector3D footPos = d_side == Types::RIGHT ? leftFootPos : rightFootPos;
  Vector3D delta = ballPos - footPos;
  _debugLevel4(ident() << " Delta: " << delta << endl);

  // See if we can kick
  bool canWeKick =  delta[1] >= params.yRange.first && delta[1] <= params.yRange.second
      && delta[0] >= params.xRange.first && delta[0] <= params.xRange.second;
  
 	_debugLevel1("canWeKick: " << capable(canWeKick).first);       

 	return capable(canWeKick);
}
