#include "kickto.ih"

Vector3D KickTo::calculateMaxGap(vector<Vector3D> players, Vector3D goal1Pos, Vector3D goal2Pos)
{ 
  Vector3D leftShoulders[players.size()];
  Vector3D rightShoulders[players.size()];

  _debugLevel4( "calculate Max gap");

  //calculate shoulders of obstructing players
  for (size_t i = 0; i < players.size(); ++i)
  {
    Vector3D shoulderVec = players[i].normalize(); 
    shoulderVec.setZ(0);
    shoulderVec = shoulderVec.crossProduct(Vector3D(0,0,1));
    
    leftShoulders[i] = players[i] - shoulderVec*1.4; //shoulder is size 1.4
    rightShoulders[i] = players[i] + shoulderVec*1.4;
    
  }
  
  float goalAngle = goal1Pos.angle(goal2Pos);
 
  vector<Shoulder> shoulders;
  Shoulder shoulder;
  shoulder.side = 'R';
  shoulder.pos = goal1Pos;
  shoulder.angle = 0;
  shoulders.push_back(shoulder);
  
  
  
  //fill shoulders. goal1Pos is used as a right shoulder & goal2Pos is used as a left shoulder
  for (size_t idx = 0; idx < players.size()*2 ; ++idx)
  {
    shoulder.side = (idx < players.size()) ? 'L' : 'R';
    shoulder.pos = (idx < players.size() ) 
                      ? leftShoulders[idx] 
                      : rightShoulders[idx - players.size()];
                      
    //if shoulder is left of goalpost it gets negative angle                 
    shoulder.angle = goal2Pos.angle(shoulder.pos) > goalAngle 
                      ? -1 * goal1Pos.angle(shoulder.pos) 
                      : goal1Pos.angle(shoulder.pos); 
    shoulders.push_back(shoulder);
    
    //_debugLevel2(" shoulder"<< idx << " " << goal1Pos.angle(shoulder.pos) << " " << goal2Pos.angle(shoulder.pos));
    //_debugLevel2("pos " << shoulder.pos.getX() << " " << shoulder.pos.getY() << " " << shoulder.pos.getZ());
  }
  shoulder.side = 'L';
  shoulder.pos = goal2Pos;  
  shoulder.angle = goalAngle;
  shoulders.push_back(shoulder);
      
      /*
  for (size_t i = 1; i < 3 ; i++){
   _debugLevel2(": " << shoulders[i].side << " " << shoulders[i].angle << " " << goalAngle);
   _debugLevel2(": " << goal1Pos.angle(shoulders[i].pos) << " " << goal2Pos.angle(shoulders[i].pos));
   }
      */
  //sort shoulder, left to right
  sort(shoulders.begin(), shoulders.end());  
    
  /*for (size_t i = 0 ; i < shoulders.size() ; i++)
   _debugLevel1("after sort: " << shoulders[i].side << " " << shoulders[i].angle);*/
    
  bool gapFound = false;  
  float maxGapSize = -1;
  Vector3D curBestTarget, curGapStart;
  int shoulderCounter = -1; 
      
  for (size_t i = 0; i < shoulders.size(); ++i)
  {
    Shoulder curShoulder = shoulders[i];     
    shoulderCounter += (shoulders[i].side == 'R') ? 1 : -1;
       
    if (shoulderCounter == 0)
      curGapStart = curShoulder.pos;
    
    if (curShoulder.side == 'L' && shoulderCounter == -1 && curGapStart.isValid())
    {
      gapFound = true;
      float gapSize = curShoulder.pos.angle(curGapStart);
            _debugLevel4("Goal pos: " << goal1Pos.getX() << " " << goal1Pos.getY() << " " << goal2Pos.getX() << " " << goal2Pos.getY() );
      _debugLevel1("GAP FOUND pos: " << ((curGapStart + curShoulder.pos) / 2).getX() << " " << ((curGapStart + curShoulder.pos) / 2).getY() );
      
      if (gapSize > maxGapSize)
      { 
        maxGapSize = gapSize;
        curBestTarget = (curGapStart + curShoulder.pos) / 2;
      }
      
      //curGapStart = 0;
    }
  }
  
  if (!gapFound){
    _debugLevel2("No gap found! Let's bowl!");
    return (goal1Pos + goal2Pos) / 2;
  }
  
  return curBestTarget;
}

/*
 *  Returns an array with the positions of all the players who are between you and the goal
 */
vector<Vector3D> KickTo::getObstructingPlayers(vector<Vector3D> allPlayers, size_t noOfPlayers, Vector3D goal1Pos, Vector3D goal2Pos)
{
  float goalAngle = goal1Pos.angle(goal2Pos);
  vector<Vector3D> obstructingPlayers;
  
  for(size_t idx = 0; idx < allPlayers.size(); ++idx)
  {
    allPlayers[idx].setZ(0);
    float angleLeftPostToPlayer = goal1Pos.angle(allPlayers[idx]);
    float angleRightPostToPlayer = goal2Pos.angle(allPlayers[idx]);
    
      _debugLevel3("goalAngle: " << goalAngle << " goalPos1 2Player: " << angleLeftPostToPlayer << "goalPos2: " << angleRightPostToPlayer);
    
    if (angleLeftPostToPlayer <= goalAngle && angleRightPostToPlayer <= goalAngle)
    {
      _debugLevel2("Obstructed by player: " << idx << " pos: " << allPlayers[idx].getX() << " " << allPlayers[idx].getY());
      obstructingPlayers.push_back(allPlayers[idx]);
    }
  }
 
  return obstructingPlayers;
}

Vector3D KickTo::calculateTarget()
{
  WorldModel &wm = SWorldModel::getInstance();
  const int NO_OF_OPPONENTS = wm.getNumberOfOpponents(); 
  const int NO_OF_TEAM_MEMBERS = wm.getNumberOfPlayers();

  _debugLevel3("Opponents: " <<  NO_OF_OPPONENTS << " Team members: " << NO_OF_TEAM_MEMBERS);  
  vector<Vector3D> players;

  _debugLevel3("getting goal1Pos and goal2Pos");  
  Vector3D goal1Pos = wm.getObject(Types::GOAL1THEM).position.getMu();
  Vector3D goal2Pos = wm.getObject(Types::GOAL2THEM).position.getMu();
  Vector3D ownPos = wm.getObject(Types::SELF).position.getMu();
  
  goal1Pos.setZ(0);
  goal2Pos.setZ(0);
  
  if (NO_OF_OPPONENTS + NO_OF_TEAM_MEMBERS == 0)
    return (goal1Pos + goal2Pos) / 2;
  
  for (int i(Types::OPPONENT1); i < ((int)Types::OPPONENT1 + NO_OF_OPPONENTS); ++i)
  {
    players.push_back(wm.getObject(static_cast<Types::Object>(i)).position.getMu());  
   }
    
  for (int i(Types::PLAYER1); i < ((int)Types::PLAYER1 + NO_OF_TEAM_MEMBERS); ++i)
  {
    Vector3D tempPos = wm.getObject(static_cast<Types::Object>(i)).position.getMu();  
    
    //You don't have to check whether you are in your own way
    if (tempPos != ownPos)
    {
      players.push_back(tempPos);
    }
  }
    _debugLevel2("getting obstructing players - total Players: " << players.size());
  vector<Vector3D> obstructingPlayers = getObstructingPlayers(players, NO_OF_TEAM_MEMBERS + NO_OF_OPPONENTS, goal1Pos, goal2Pos);
  
  _debugLevel2(" obstructing players: " << obstructingPlayers.size());
  
  if (obstructingPlayers.size() == 0)
    return (goal1Pos + goal2Pos) / 2;
  
     _debugLevel2("calculating max gaps - obstructing players: " << obstructingPlayers.size()); 
  Vector3D target = calculateMaxGap(obstructingPlayers, goal1Pos, goal2Pos);
      
  return target;   
}
