#include "kickto.ih"

rf<Behavior::Goal> KickTo::generateGoal(unsigned step, unsigned slot)
{
  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();
   
  WorldModel& wm = SWorldModel::getInstance();

  if (step == 1)
  {
    Vector3D ballPos = wm.getObject(Types::BALL).position.getMu();
    ballPos.setZ(0);
    Vector3D goal1Pos = wm.getObject(Types::GOAL1THEM).position.getMu();
    Vector3D goal2Pos = wm.getObject(Types::GOAL2THEM).position.getMu();
    
    //calculates a new target to kick the ball to once every 25 times.
    //if a new target is calculate to often the positioning of the agent will be put under a lot of pressure
    if (d_timeCounter % 25 == 0)
    {
      d_goalPos = calculateTarget();
      d_firstTime = false;
      d_timeCounter = 0;      
    }

    ++d_timeCounter;
    
    //to prevent the agent from repositioning all the time the calculated target will be
    //changed into the closest predetermined fixed targets
    vector<Vector3D> fixedGoalPositions; 
    fixedGoalPositions.push_back((goal1Pos * 4 + goal2Pos)/5);
    fixedGoalPositions.push_back((goal1Pos * 3 + goal2Pos * 2)/5);
    fixedGoalPositions.push_back((goal1Pos + goal2Pos)/2);
    fixedGoalPositions.push_back((goal1Pos * 2+ goal2Pos * 3)/5);
    fixedGoalPositions.push_back((goal1Pos + goal2Pos * 4)/5);
    
    float smallestAngle = 10000.0, curAngle;
    Vector3D bestPosMatch = Vector3D(0,0,0);
    
    for (size_t i=0; i < fixedGoalPositions.size(); ++i)
    {
      curAngle = d_goalPos.angle(fixedGoalPositions[i]);
      if (curAngle < smallestAngle)
      {
        smallestAngle = curAngle;
        bestPosMatch = fixedGoalPositions[i];
      }
    }
     
    d_goalPos = bestPosMatch;
    
    Vector3D goalXYPos = d_goalPos;
    
    goalXYPos.setZ(0);
    Vector3D straight(0,1,0);
    
    double angle = straight.angle(goalXYPos);
    
    if (goalXYPos.getX() > 0)
      angle = -angle;
      
    Vector3D dir = d_goalPos - ballPos;

    Vector3D nDir = dir.normalize();

    _debugLevel4(ballPos << " " << dir);
    //ballPos = ballPos - nDir * 0.1;//dir / dir.length() * 0.9;
    _debugLevel4(ballPos);

    // This can be negative to go for the other foot!
    Vector3D ballPosRight = nDir.crossProduct(Vector3D(0,0,1));
    //    ballPosLeft.setX(dir.getY());
    //    ballPosLeft.setY(dir.getX());
    
    _debugLevel4(goal1Pos << " " << goal2Pos << " " << d_goalPos << " " << angle);
    
    Vector3D targetPos = ballPos + ballPosRight.normalize() * (ballPos.getX() < 0 ? .5 : -.5);
    
    _debugLevel4(ballPos << " " << targetPos);
/*    if (!Math::atSameSideOf(ballPos, targetPos, Vector3D(0,0,0)))
    {
      targetPos = ballPos + ballPosLeft.normalize() * 4;
      if (!Math::atSameSideOf(ballPos, Vector3D(0,0,0), targetPos))
        targetPos = ballPos - ballPosLeft.normalize() * 4;
    }
*/
 
    con->addVar("Pos", targetPos);
    con->addVar("Angle", -angle, -angle);
    con->addVar("Dir", Vector3D(1, 0, 0));
    
    con->addVar("ThresholdAngle", d_angle_threshold,  d_angle_threshold);
  }  
  
  if (step == 2)
  {
    Vector3D goal1Pos = wm.getObject(Types::GOAL1THEM).position.getMu();
    Vector3D goal2Pos = wm.getObject(Types::GOAL2THEM).position.getMu();
    
    //just for output purposes.
    //outputs angle towards left goal post, current target and right goal post
    if (d_timeCounter++%60 == 0){
      Vector3D straight(0,1,0);
      double angle1 = straight.angle(goal1Pos)*180.0/M_PI;
      double angle2 = straight.angle(d_goalPos)*180.0/M_PI;
      double angle3 = straight.angle(goal2Pos)*180.0/M_PI;
      if (goal1Pos.getX() > 0)
        angle1 = -angle1;
      if (d_goalPos.getX() > 0)
        angle2 = -angle2;
      if (goal2Pos.getX() > 0)
        angle3 = -angle3;
      _debugLevel1("ANGLES: left post: " << angle1 << " target: " << angle2 << " right post: " << angle3);
    }
    
    Vector3D goalXYPos = d_goalPos;
    goalXYPos.setZ(0);
    
    con->addVar("Goal", goalXYPos);
  }
  // con->addVar("Var", 0, 1);
  
  return goal;
}
