/*
 *  Little Green BATS (2007), AI department, University of Groningen
 *
 *  Authors: 	Martin Klomp (martin@ai.rug.nl)
 *		Mart van de Sanden (vdsanden@ai.rug.nl)
 *		Sander van Dijk (sgdijk@ai.rug.nl)
 *		A. Bram Neijt (bneijt@gmail.com)
 *		Matthijs Platje (mplatje@gmail.com)
 *
 *	All students of AI at the University of Groningen
 *  at the time of writing. See: http://www.ai.rug.nl/
 *
 *  Date: 	July 27, 2007
 *
 *  Website:	http://www.littlegreenbats.nl
 *
 *  Comment:	Please feel free to contact us if you have any 
 *		problems or questions about the code.
 *
 *
 *  License: 	This program is free software; you can redistribute 
 *		it and/or modify it under the terms of the GNU General
 *		Public License as published by the Free Software 
 *		Foundation; either version 3 of the License, or (at 
 *		your option) any later version.
 *
 *   		This program is distributed in the hope that it will
 *		be useful, but WITHOUT ANY WARRANTY; without even the
 *		implied warranty of MERCHANTABILITY or FITNESS FOR A
 *		PARTICULAR PURPOSE.  See the GNU General Public
 *		License for more details.
 *
 *   		You should have received a copy of the GNU General
 *		Public License along with this program; if not, write
 *		to the Free Software Foundation, Inc., 59 Temple Place - 
 *		Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef _BATS_KICKTO_HH_
#define _BATS_KICKTO_HH_

#include "behavior.hh"
#include "bufferedif.h"

namespace bats
{
  /**
   *  Behavior: Kick the ball to the goal
   */
  class KickTo : public Behavior
  {
    Vector3D d_goalPos;
    bool d_firstTime;
    unsigned d_timeCounter;
    
    double d_max_distance, d_angle_threshold;

    BufferedIf d_buffered_if;
    virtual rf<Goal> generateGoal(unsigned step, unsigned slot);
    virtual rf<State> getCurrentState();

    /** Get Capability
     *
     *  @param s Expected state variable: None
     *  @param g Expected goal variable: None
     *  @returns confidence (0.5, 0.2) when are at the right position to kick to the goal.
     */
    virtual ConfidenceInterval getCapability(rf<State> s, rf<Goal> g);
  
  public:
    

    
    struct Shoulder
    {
      Vector3D pos;
      char side;
      float angle;
      
      bool operator<(Shoulder const &other) const
      {
        return angle < other.angle;
      }
      
    };
    
//    bool compareAngles (Shoulder i,Shoulder j) { return (i<j); }
  
  
    KickTo(std::string const &id, std::string const &playerClass);
    Vector3D calculateTarget();
    
  private:
    std::vector<Vector3D> getObstructingPlayers(std::vector<Vector3D> allPlayers, size_t noOfPlayers, Vector3D goal1Pos, Vector3D goal2Pos);
    Vector3D calculateMaxGap(std::vector<Vector3D> players, Vector3D goal1Pos, Vector3D goal2Pos);
  };
};

#endif
