#include "movejointrubto.ih"

rf<Behavior::Goal> MoveJointRubTo::generateGoal(unsigned step, unsigned slot)
{
  /*
  For rubbery movement we have a constant force which will have a speed effect
  which is based on the difference from the complete angle.
  As the angle decreses, the force will decrease because the same force will
  have less effect on the angle as it get's closer to it's minimum difference.

  What formula is for a spring model again???? DUnno anymore...
  Well, let's just say: squared (most things in nature are anyway)

  The force of the spring is squared with the distance ->
  speed = da^2
  
  Doesn't work: speed of 0 still won't make the naobot collapse in rcsserver 0.6
  */

	//WorldModel& wm = SWorldModel::getInstance();


	rf<Goal> goal = new Goal();
	rf<OrNode> dis = goal->addDisjunct();
	rf<AndNode> con = dis->addConjunct();
	AgentModel& am = SAgentModel::getInstance();

	
	// Get our own goal.
	rf<StateVarNode> angleNode = rf_cast<StateVarNode>(d_goal->findDeep("Angle"));
	rf<StateVarNode> speedNode = rf_cast<StateVarNode>(d_goal->findDeep("Speed"));

	double goalAngle;
	double speed;
	
	if (!angleNode && speedNode)
	{
		speed = speedNode->getVar().second.mean();
	}
	else
	{
		if (angleNode)
			goalAngle = angleNode->getVar().second.mean();
		else
			goalAngle = am.getJoint(d_joint)->angle.getMu();

		rf<StateVarNode> maxSpeedNode = rf_cast<StateVarNode>(d_goal->findDeep("MaxSpeed"));
		double maxSpeed = 0;
	
		if (maxSpeedNode)
		{
			Range maxSpeedRange = maxSpeedNode->getVar().second;
			maxSpeed = maxSpeedRange.mean();
		}

		NormalDistribution1D angle = am.getJoint(d_joint)->angle;
		//double da = Math::normalizeRad(goalAngle - angle.getMu());
		double da = goalAngle - angle.getMu();
	
		rf<StateVarNode> gainNode = rf_cast<StateVarNode>(d_goal->findDeep("Gain"));
		double gain = d_gain;
	
		if (gainNode)
			gain = gainNode->getVar().second.mean();
	
	//#ifdef EXPSPEED	
	//	double speed = (exp(da) - 1) * gain;
	//#else
	//	double speed = gain * da;
//	#endif

		speed = da*fabs(da);
		//speed = 0.0001;
	//double minspeed = 0.005;
	//if(speed < minspeed && speed > -minspeed)
	//	speed = 0.0;
	//cout << speed << "\n";
	
		if (maxSpeed > 0)
		{
			if (speed > maxSpeed)
				speed = maxSpeed;
			
			if (speed < -maxSpeed)
				speed = -maxSpeed;
		}
		/*
		else if (maxSpeed < 0)
			speed *= 2;
    */
		_debugLevel4(ident() << ": " << goalAngle << " " << angle.getMu() << " " << da << " " << speed << " " << maxSpeed);
	}

	con->addVar("Speed", speed, speed);
	con->addVar("Joint", d_joint, d_joint);
	
	return goal;
}

