
#include "pass.ih"

namespace {

bool stupidRobot(Pass::Target const &t)
{
  return t.direction.getY() < 0 || t.location.getMu().length() > 3.5 || t.location.getMu().length() < 1;
}

}

void Pass::getTargetsInto(std::vector<Pass::Target> *targetList) const
{
  WorldModel& wm = SWorldModel::getInstance();

  targetList->clear();

  //Find all possible pass targets and place them in a list
  for (size_t i = 0; i < 5; ++i)
  {
    Types::Object player =  static_cast<Types::Object>(i + Types::PLAYER1);
    if (wm.isObjectAlive(player))
    {
      Target target;
      target.location = wm.getObject(player).position;
      target.direction = Math::cartesianToPolar(target.location.getMu(), true);
      target.safeAngle.first = -M_PI;
      target.safeAngle.second = M_PI;
      targetList->push_back(target);
    }
  }

  //Targets must be in front of the robot, between 0 and PI
  std::vector<Target>::iterator l = remove_if(targetList->begin(), targetList->end(), stupidRobot);
  targetList->erase(l, targetList->end());

  //Apply Calculate the right free angle to all targets
  //If an opponent is within the radius of the passable target
  // then it is considered a threat and should be used
  // to minimize the safeAngle.
  for (size_t i = 0; i < 5; ++i)
  {
    Types::Object opponent =  static_cast<Types::Object>(i + Types::OPPONENT1);
    if (wm.isObjectAlive(opponent))
    {
      NormalDistribution3D oppPosition = wm.getObject(opponent).position;
      Vector3D oppPositionPolar = Math::cartesianToPolar(oppPosition.getMu(), true);

      //If opponent is behind us, don't bother
      if(oppPositionPolar.getY() < 0)
        continue;

      _foreach(target, *targetList)
      {
        //If within radius
        //TODO Add a scare factor
        if(target->direction.getX() < oppPositionPolar.getX())
          continue;

        //If the opponent is more left then our target and it's angle is smaller then we currently have for the left, change it
        if(target->direction.getY() < oppPositionPolar.getY() && target->safeAngle.first > oppPositionPolar.getY())
          target->safeAngle.first = oppPositionPolar.getY();

        //Opponent is right of our target and our safeAngle does not contain his angle yet
        if(target->direction.getY() > oppPositionPolar.getY() && target->safeAngle.second > oppPositionPolar.getY())
          target->safeAngle.second = oppPositionPolar.getY();
      }
    }
  }
}
