#ifndef _BATS_PERIODIC_JOINT_HH_
#define _BATS_PERIODIC_JOINT_HH_

#include "Conf/xmlnode.hh"

namespace bats
{
	class PeriodicJoint
	{
		typedef double (PeriodicJoint::*ModulationFunc)();
	
		double d_modulationSpan;
		double d_startTime;
		double d_initTime;
		double d_delay;
		double d_period;
		double d_amplitudeStart;
		double d_frequencyStart;
		double d_prevFrequencyTime;
		double d_prevSimuTime;

		ModulationFunc d_factor;
	
		ModulationFunc d_amplitude;
		ModulationFunc d_sampleTime;

		public:
			PeriodicJoint(double period = 1.0, double delay = 0.0, double modulationSpan = 0.0);
		
			void enableAmplitudeModulation(double start);
			void enableFrequencyModulation(double start);
		
			bool active();
		
			double amplitude();
			double sampleTime();
		
			// Properties
			void setPeriod(double period) { d_period = period; };
			void setDelay(double delay) { d_delay = delay; };
			void setModulationSpan(double modulationSpan) { d_modulationSpan = modulationSpan; };
		
			double period() { return d_period; };
			double delay() { return d_delay; };
			double modulationSpan() { return d_modulationSpan; };
			
			double amplitudeStart() { return d_amplitudeStart; };
			double frequencyStart() { return d_frequencyStart; };
			
			bool amplitudeModulation() const { return d_amplitude == &PeriodicJoint::factor; };
			bool frequencyModulation() const { return d_sampleTime == &PeriodicJoint::modulateTime; };
			
			void reset();
		private:
			double factor();
			double factorNoop();
		
			double modulateTime();
			double modulateTimeNoop();
		
			double modulateTimeReal(double factor);
	};

	inline double 
	PeriodicJoint::factorNoop()
	{
		return 1.0;
	}

	inline double
	PeriodicJoint::amplitude()
	{
		return d_amplitudeStart + (1 - d_amplitudeStart) * (this->*d_amplitude)();
	}

	inline double
	PeriodicJoint::sampleTime()
	{
		return (this->*d_sampleTime)();
	}

	inline double
	PeriodicJoint::modulateTimeNoop()
	{
		return modulateTimeReal(1.0);
	}

	inline void
	PeriodicJoint::enableAmplitudeModulation(double start)
	{
		d_amplitude = &PeriodicJoint::factor;
		d_amplitudeStart = start;
	}

	inline void
	PeriodicJoint::enableFrequencyModulation(double start)
	{
		d_sampleTime = &PeriodicJoint::modulateTime;
		d_frequencyStart = start;
	}
}

#endif /* _BATS_PERIODIC_JOINT_HH_ */
