#ifndef _BATS_PLAYMODE_HH_
#define _BATS_PLAYMODE_HH_

#include "behavior.hh"
#include "../../Types/types.hh"

namespace bats
{
  /** Behavior that is only capable is any of the given gamestates are active
    This behavior is transparent, the previously used goal in getCapability is always used as
    a goal for the generateGoal
    
    Configuration is done using the playmodes parameter and playmode names from types. case sensitive:
    <behavior type="PlayMode" id="win">
        <param>
          <playmodes>UNKNOWN KICKIN_LEFT</playmodes>
        </param>
        <slot index="0-0">
          <behavior refid="keeperfall_left_sp"/>
        </slot>
      </behavior>
   To negate the matching you can also add a "not="1"" attribute to the playmodes element
  */
  class PlayMode : public Behavior
  {
    rf<Goal> d_goal;
    bool d_not;
    std::vector<Types::PlayMode> d_modes;
    
    virtual rf<Goal> generateGoal(unsigned step, unsigned slot);
    virtual rf<State> getCurrentState();
    virtual ConfidenceInterval getCapability(rf<State> s, rf<Goal> g);
  
  public:
    PlayMode(std::string const &id, std::string const &playerClass);
  };
};

#endif
