#include "score.ih"

rf<Behavior::Goal> Score::generateGoal(unsigned step, unsigned slot)
{
  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();
  
  WorldModel& wm = SWorldModel::getInstance();

  vector<PlayerInfo> allPlayers;
  for (WorldModel::player_iterator iter = wm.beginPlayers(); iter != wm.endPlayers(); ++iter)
  {
    allPlayers.push_back(iter->second);
  }
  for (WorldModel::player_iterator iter = wm.beginOpponents(); iter != wm.endOpponents(); ++iter)
  {
    allPlayers.push_back(iter->second);
  }

  
  Vector3D ballPos = wm.getObject(Types::BALL).position.getMu();
  ballPos[2] = 0;
  Vector3D goal1Pos = wm.getObject(Types::GOAL1THEM).position.getMu();
  goal1Pos[2] = 0;  
  Vector3D goal2Pos = wm.getObject(Types::GOAL2THEM).position.getMu();
  goal2Pos[2]= 0;
  Vector3D curBestTarget = (goal1Pos + goal2Pos) / 2; // just in case there is no place to kick


  _debugLevel4("Determine best shot angle");

  float goalAngle = (goal2Pos - ballPos).angle(goal1Pos - ballPos);
 
  vector<Shoulder> shoulders;
  Shoulder shoulder;
  shoulder.side = 'R';
  shoulder.pos = goal1Pos;
  shoulder.angle = 0;
  shoulders.push_back(shoulder);

  for (vector<PlayerInfo>::iterator iter = allPlayers.begin(); iter != allPlayers.end(); ++iter)
  {
    PlayerInfo info = *iter;
    Vector3D playerPos = info.getPosition();
    playerPos[2] = 0;

    playerPos = (playerPos - ballPos).crossProduct(Vector3D(0,0,1)).normalize();

    for (size_t indx = 0; indx < 2; ++indx)
    {
      shoulder.side = (indx == 0) ? 'L' : 'R';
      shoulder.pos = (indx == 0) ? (info.getPosition() - playerPos * 0.05) : (info.getPosition() + playerPos * 0.05);
      shoulder.angle = (goal1Pos - ballPos).angle(shoulder.pos - ballPos);
      shoulders.push_back(shoulder);
    }
  }

  shoulder.side = 'L';
  shoulder.pos = goal2Pos;  
  shoulder.angle = goalAngle;
  shoulders.push_back(shoulder);

  //sort shoulder, left to right
  sort(shoulders.begin(), shoulders.end());  


  for (size_t i = 0 ; i < shoulders.size() ; i++)
   _debugLevel1("after sort: " << shoulders[i].side << " " << shoulders[i].angle);


  bool gapFound = false;  
  float maxGapSize = -1;
  Vector3D curGapStart;
  int shoulderCounter = -1; ;

 for (size_t i = 0; i < shoulders.size(); ++i)
  {
    Shoulder curShoulder = shoulders[i];     
    shoulderCounter += (shoulders[i].side == 'R') ? 1 : -1;
       
    if (shoulderCounter == 0)
      curGapStart = curShoulder.pos;
    
    if (curShoulder.side == 'L' && shoulderCounter == -1 && curGapStart.isValid())
    {
      gapFound = true;
      float gapSize = curShoulder.pos.angle(curGapStart);
      _debugLevel4("Goal pos: " << goal1Pos.getX() << " " << goal1Pos.getY() << " " << goal2Pos.getX() << " " << goal2Pos.getY() );
      _debugLevel1("GAP FOUND pos: " << ((curGapStart + curShoulder.pos) / 2).getX() << " " << ((curGapStart + curShoulder.pos) / 2).getY() );

      if (gapSize > maxGapSize)
      { 
        maxGapSize = gapSize;
        curBestTarget = (curGapStart + curShoulder.pos) / 2;
      }
      
      //curGapStart = 0;
    }
  }



  Vector3D goalPos = curBestTarget;
  Vector3D dir = (curBestTarget - ballPos).normalize();

  if (step == 0)
  {
    Vector3D targetPos = ballPos - dir * d_ballDistY;
    Vector3D ballPosRight = dir.crossProduct(Vector3D(0,0,1)).normalize();
    _debugLevel1("Dir: " << dir << ", left: " << ballPosRight);
    _debugLevel1("x: " << d_ballDistX << ", target: " << targetPos);
    targetPos = targetPos + ballPosRight * (targetPos[0] > 0 ? -1 : 1) * d_ballDistX;
    _debugLevel1("target: " << targetPos);
    
    con->addVar("Pos", targetPos);
    con->addVar("Dir", goalPos - ballPos);
  }
  else
  {
    con->addVar("Dir", goalPos - ballPos);
  }
  
  // con->addVar("Var", 0, 1);
  
  return goal;
}
