#include "score.ih"
#include "avoider.h"
namespace {
bool lookingAtGoal(float l, float r)
{
	return (l > M_PI/2.0 - 0.1 && r < M_PI/2.0 + 0.1);
}
}


Behavior::ConfidenceInterval Score::getCapability(rf<State> s, rf<Goal> g)
{
  WorldModel& wm = SWorldModel::getInstance();
  AgentModel& am = SAgentModel::getInstance();
  
  //Marijn:
  if (g->findDeep("SurpressScore"))
    return ConfidenceInterval(0, 0);
  
  Vector3D ballPos = wm.getObject(Types::BALL).position.getMu();
  Vector3D goalPos = (wm.getObject(Types::GOAL1THEM).position.getMu() + wm.getObject(Types::GOAL2THEM).position.getMu()) / 2.0;
  Vector3D ballToGoalVec = goalPos - ballPos;
  Vector3D dir = (goalPos - ballPos).normalize();
  
  float minDistOppToBall = 99999.9;
  for (WorldModel::player_iterator iter = wm.beginOpponents(); iter != wm.endOpponents(); ++iter)
  {
    Vector3D opp = (iter->second).getPosition();
    if (minDistOppToBall > (ballPos - opp).length())
    	minDistOppToBall = (ballPos - opp).length();
  }
  
  //General stuff:
  if (am.onMyBack() || am.onMyBelly() || am.onMySide() || wm.getPlayMode() == Types::GOAL_KICK_US) 
    return capable(false);
    
  ballPos[2] = 0;
  
  // Dont panic if we are looking at our own goal
  float ownGoalAngle1 = Math::cartesianToPolar(wm.getObject(Types::FLAG1US).position.getMu(), true).getY();
  float ownGoalAngle2 = Math::cartesianToPolar(wm.getObject(Types::FLAG2US).position.getMu(), true).getY();
  _debugLevel1("Goalangle1: "<< ownGoalAngle1 << " goalangle2: " << ownGoalAngle2);
  if (lookingAtGoal(ownGoalAngle2, ownGoalAngle1)){
  	_debugLevel1("Owngoal in front, do score!");
    return capable(true);
  }
    
     
  //Strategic stuff:
  if (ballToGoalVec.length() > d_max_distance || (minDistOppToBall < d_min_opp_distance))
  	return ConfidenceInterval(0.55, 0.0); //capable(medium)
	/*
	//If we are close to the goal, then don't score. This should force dribble to take over and simply
	//  dribble the ball into the goal. HACK
	
	//One of the goal pole is close enough
	if (ballToGoalVec.length() < d_min_distance)	  
	{	 //capable(medium)
	
		_debugLevel1("Close to goal.");
		
		//We are looking towards the other's goal
    float themGoalAngle1 = Math::cartesianToPolar(wm.getObject(Types::GOAL2THEM).position.getMu(), true).getY();
    float themGoalAngle2 = Math::cartesianToPolar(wm.getObject(Types::GOAL1THEM).position.getMu(), true).getY();
    _debugLevel1("ThemGoalangle1: "<< ownGoalAngle1 << " ThemGoalangle2: " << ownGoalAngle2);
    if (lookingAtGoal(themGoalAngle2, themGoalAngle1))
    {
      //We are pointing the right way. Check for opponents in the way
      bool frontClean = true;
      //Nobody in front of us?		 
      for (size_t i = 0; i < 5; ++i)
      {
        Types::Object player =  static_cast<Types::Object>(i + Types::OPPONENT1);
        if (wm.isObjectAlive(player))
        {
          float targetAngle = wm.getObject(player).position.getMu();
          float sAngle = M_PI/20.0; //One thenth of the side must be opponent free
          if(targetAngle > sAngle && targetAngle < M_PI - sAngle)
          {
            frontClean = false;
            break;
          }
        }
        if(frontClean)
          return capable(false);        
      }
      
      //Other's goal is in our view and close'
      return capable(true);
    }
		
		
		
		////////Even older code
		
		bool pathIsFree = true;
		
		WorldModel::player_iterator begin = wm.beginOpponents(); 
		WorldModel::player_iterator end = wm.endOpponents();
		 
		for (size_t i=0; i < 2; ++i)
		{
			for (WorldModel::player_iterator iter = begin; iter != end; ++iter)
			{
				Vector3D opp = (iter->second).getPosition();
				
				//Avoider::collides(Vector3D const &target, Vector3D const &obstacle, double radius)
				if (Avoider::collides(goalPos, opp, 0.2))
				{
					_debugLevel1("Somebody is blockin' the goal, dribble is prob. better.");
					pathIsFree = false; 
					break;
				}
			}
			
			WorldModel::player_iterator begin = wm.beginPlayers(); 
			WorldModel::player_iterator end = wm.endPlayers();
  		}
  	
  	if (pathIsFree) 
  		return capable(false);
  }
*/
	
  return capable(true);
}
