/*
 *  Little Green BATS (2007), AI department, University of Groningen
 *
 *  Authors: 	Martin Klomp (martin@ai.rug.nl)
 *		Mart van de Sanden (vdsanden@ai.rug.nl)
 *		Sander van Dijk (sgdijk@ai.rug.nl)
 *		A. Bram Neijt (bneijt@gmail.com)
 *		Matthijs Platje (mplatje@gmail.com)
 *
 *	All students of AI at the University of Groningen
 *  at the time of writing. See: http://www.ai.rug.nl/
 *
 *  Date: 	July 27, 2007
 *
 *  Website:	http://www.littlegreenbats.nl
 *
 *  Comment:	Please feel free to contact us if you have any 
 *		problems or questions about the code.
 *
 *
 *  License: 	This program is free software; you can redistribute 
 *		it and/or modify it under the terms of the GNU General
 *		Public License as published by the Free Software 
 *		Foundation; either version 3 of the License, or (at 
 *		your option) any later version.
 *
 *   		This program is distributed in the hope that it will
 *		be useful, but WITHOUT ANY WARRANTY; without even the
 *		implied warranty of MERCHANTABILITY or FITNESS FOR A
 *		PARTICULAR PURPOSE.  See the GNU General Public
 *		License for more details.
 *
 *   		You should have received a copy of the GNU General
 *		Public License along with this program; if not, write
 *		to the Free Software Foundation, Inc., 59 Temple Place - 
 *		Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef _BATS_SCORE_HH_
#define _BATS_SCORE_HH_

#include "behavior.hh"

namespace bats
{

  /**
   *  Behavior: Tries to score
   *
   *  Compatible subbehaviors:
   *  -# index 0-0: WalkTo (Gets a Pos goal).
   *  -# index 1-0: KickTo (Just kicks the ball towards the goal).
   */
  class Score : public Behavior
  {
    double d_ballDistX;
    double d_ballDistY;
    
    double d_max_distance;
    double d_min_opp_distance;
    double d_min_distance;

    int d_last_time;
    int d_goal_no;
    double d_max_time;

    virtual rf<Goal> generateGoal(unsigned step, unsigned slot);
    virtual rf<State> getCurrentState();
    
    /** Get Capability
     *
     *  @param s Expected state variable: None
     *  @param g Expected goal variable: None
     *  @returns confidence (0.5, 0.2) when standing, otherwise (0.0, 0.0).
     */
    virtual ConfidenceInterval getCapability(rf<State> s, rf<Goal> g);
  
  public:

    struct Shoulder
    {
      Vector3D pos;
      char side;
      float angle;
      
      bool operator<(Shoulder const &other) const
      {
        return angle < other.angle;
      }
      
    };

    Score(std::string const &id, std::string const &playerClass);

  };
};

#endif
