#include "sine.ih"

Sine::Sine(string const &id, string const &playerClass)
:
	Behavior("Sine", id, playerClass),
	d_jointparams(Types::NJOINTS),
	d_lastTime(0.0),
  d_smoothTransfer(false),
  d_startupTime(0.0),
  d_shutdownTime(0.0),
  d_factor(0.0),
  d_lastUpdate(-1)
{
  _debugLevel1("New sine: " << id);
  
  for (unsigned i = 0; i < Types::NJOINTS; ++i)
    d_jointparams[i] = new JointParams();

  d_smoothTransfer = Behavior::getParam("/smoothtransfer",string("0")) == "1";
  _debugLevel3(ident() << "Smooth transfer: " << (d_smoothTransfer ? "yes" : "no"));
	XMLNodeSet params = Behavior::getParams("/settings");
	istringstream paramReader;
	
	if (params && !params.empty())
	{
		XMLNode paramNode = params.front();
		paramReader.str(paramNode.getContent());

		while (!paramReader.eof())
		{
			_debugLevel4("Reading new line..");
			unsigned joint;
			char punc;

			paramReader >> joint >> punc;
			
			if (punc != ':')
			{
				_debugLevel4("Wrong format: '" << punc << "', expected ':' (if this is the last line, it's OK)");
				_debugLevel4("Last read joint: " << joint);
				_debugLevel4(paramNode.getContent());
				break;
			}
        
			_debugLevel4("For joint " << joint);
			
			double period = 0;

 			while (punc != ';')
			{
				SineParams parameters;
				paramReader >> parameters.amplitude >> parameters.period >> parameters.phase >> parameters.offset >> punc;

				if (punc == '#')
				{
					double delay;
					paramReader >> delay >> punc;
					
					d_jointparams[joint]->joint.setDelay(delay);
					_debugLevel4("StartDelay = " << d_jointparams[joint]->joint.delay() << " punc = " << punc);
				}
				
				if (parameters.period > period)
					period = parameters.period;

				d_jointparams[joint]->sines.push_back(parameters);
			}
			
			d_jointparams[joint]->joint.setPeriod(period);
		}
	}
	else
	{
		_debugLevel4("no settings parameters found");
	}

	d_startupTime = Behavior::getParam("/startupTime", 1.0);
	d_shutdownTime = Behavior::getParam("/shutdownTime", 1.0);

	double frequencyStart = Behavior::getParam("/frequencyStart", -1.0);
	double amplitudeStart = Behavior::getParam("/amplitudeStart", -1.0);
	
	_debugLevel4("startUp: " << d_startupTime << "\n freqStart: " << frequencyStart << " \namplitudeStart: " << amplitudeStart);
	
	if (d_startupTime != 0.0)
	{
		for (vector<rf<JointParams> >::iterator iter = d_jointparams.begin(); iter != d_jointparams.end(); ++iter)
		{
			(*iter)->joint.setModulationSpan(d_startupTime);
			
			if (frequencyStart >= 0)
				(*iter)->joint.enableFrequencyModulation(frequencyStart);
			
			if (amplitudeStart >= 0)
				(*iter)->joint.enableAmplitudeModulation(amplitudeStart);
		}
	}
	
	d_tree = new AST::Node(sequenceType);
	d_tree->addChild(new AST::Node(andType));
	
	for (unsigned i = 0; i < Types::NJOINTS; ++i)
		d_tree->getChild(0)->addChild(new AST::Node(orType));
    
	s_sines.push_back(this);
}

