/*
 *  Little Green BATS (2007), AI department, University of Groningen
 *
 *  Authors: 	Martin Klomp (martin@ai.rug.nl)
 *		Mart van de Sanden (vdsanden@ai.rug.nl)
 *		Sander van Dijk (sgdijk@ai.rug.nl)
 *		A. Bram Neijt (bneijt@gmail.com)
 *		Matthijs Platje (mplatje@gmail.com)
 *
 *	All students of AI at the University of Groningen
 *  at the time of writing. See: http://www.ai.rug.nl/
 *
 *  Date: 	July 27, 2007
 *
 *  Website:	http://www.littlegreenbats.nl
 *
 *  Comment:	Please feel free to contact us if you have any 
 *		problems or questions about the code.
 *
 *
 *  License: 	This program is free software; you can redistribute 
 *		it and/or modify it under the terms of the GNU General
 *		Public License as published by the Free Software 
 *		Foundation; either version 3 of the License, or (at 
 *		your option) any later version.
 *
 *   		This program is distributed in the hope that it will
 *		be useful, but WITHOUT ANY WARRANTY; without even the
 *		implied warranty of MERCHANTABILITY or FITNESS FOR A
 *		PARTICULAR PURPOSE.  See the GNU General Public
 *		License for more details.
 *
 *   		You should have received a copy of the GNU General
 *		Public License along with this program; if not, write
 *		to the Free Software Foundation, Inc., 59 Temple Place - 
 *		Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef _BATS_SINE_HH_
#define _BATS_SINE_HH_

#include "worldmodel.hh"
#include "behavior.hh"
#include "periodicjoint.hh"
#include "periodicinterface.hh"

namespace bats
{
	class Sine : public Behavior, public PeriodicInterface
	{
		public:
			struct SineParams
			{
			  SineParams()
			    :
			    amplitude(0),
			    period(0),
			    phase(0),
			    offset(0)
			  {}
				double amplitude;
				double period;
				double phase;
				double offset;
			};
    
			struct JointParams : public RefAble
			{
				JointParams()
        :
          joint(),
          sines(),
          startTime(0),
          freqfactor(1)
        {}
				
				PeriodicJoint joint;
				std::vector<SineParams> sines;

        double startTime;
				double freqfactor; 
			};
    
    protected:
			std::vector<rf<JointParams> > d_jointparams;
    
			double d_lastTime;
      bool d_smoothTransfer;
      double d_startupTime;
      double d_shutdownTime;
      double d_factor;
      double d_lastUpdate;
      
			virtual double calcAngle(unsigned slot);

      virtual void preGenerateGoal(unsigned step);
      
			virtual rf<Goal> generateGoal(unsigned step, unsigned slot);
			virtual rf<State> getCurrentState() { return 0; }
			virtual ConfidenceInterval getCapability(rf<State> s, rf<Goal> g) { return ConfidenceInterval(0.5, 0.2); }

			//static std::string s_lastSineID;
			//static std::vector<std::vector<Params> > s_lastParams;

    
    /// Last time at which the previous Sine behavior ran. Saved for transition purposes
/*    static double s_lastTime;
    static double s_lastStartTime;
    static double s_lastStopTime;*/
    
    /// Last time any Sine behavior ran. Used to determine if there was a Sine at all
			static double s_lastSineTime;

			//static double s_transitionTime;

			static std::vector<rf<Sine> > s_sines;
    
		public:
			Sine(std::string const &id, std::string const &playerClass);

      double getFactor() { return d_factor < 0.0 ? 0.0 : (d_factor > 1.0 ? 1.0 : d_factor); }
      double getLastUpdate() { return d_lastUpdate < 0.0 ? SWorldModel::getInstance().getTime() - 0.02 : d_lastUpdate; }
      void setLastUpdate(double t) { d_lastUpdate = t; }
      
      void decreaseFactor(double dt) { d_factor = getFactor() - (d_shutdownTime < 0.0001 ? 1.0 : dt/d_shutdownTime); }
      void increaseFactor(double dt) { d_factor = getFactor() + (d_startupTime < 0.0001 ? 1.0 : dt/d_startupTime); }
      
			std::vector<rf<JointParams> >& getJointParams() { return d_jointparams; }

			double startPosition(Types::Joint joint);

			void reset();
			void update();
      
      static std::vector<rf<Sine> > getSines() { return s_sines; }
	};
};

#endif

