#include "stabilize.ih"

rf<Behavior::Goal> Stabilize::generateGoal(unsigned step, unsigned slot)
{
  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();
  
  if (slot == Types::LLEG2 || slot == Types::RLEG2) 
  {
    AgentModel& am = SAgentModel::getInstance();
  	WorldModel& wm = SWorldModel::getInstance();
  	
    Vector3D fieldNormal = wm.getFieldNormal();
    Vector3D gyro = am.getGyro(); 
  //  float speedOfTorso = gyro.getX();
    float angleOfTorso = fieldNormal.getY() + am.getJoint((Types::Joint)slot)->angle.getMu()/2;
    float goalSpeed = 0, goalAngle = 0;
     
    
    _debugLevel4("speedY " << gyro.getX() << "  " << gyro.getY() << " " << gyro.getZ());
  	_debugLevel4("both: " << angleOfTorso << " getY: " << fieldNormal.getY() << " jointAngle:" << am.getJoint((Types::Joint)slot)->angle.getMu());

  	//_debugLevel4(d_speedWeight << " " << d_counterAngleWeight << " " << d_counterOffsetAngle << " " << d_stabilizeThreshold);
  	
  	if (am.getGyro().length() < d_stabilizeThreshold )
  	{
  		_debugLevel1("Nearly completed. Angle: " << angleOfTorso << " speed < " << d_stabilizeThreshold);
  		d_isNearlyStabilized = true;
  	}  	
  	else if (am.getGyro().length() > d_stabilizeThreshold * 20)
  			d_isNearlyStabilized = false;
  		
  	
  	if (d_isNearlyStabilized)
  	{
	  	goalAngle = fieldNormal.getY() * d_stabilizeAngleWeight;
	  	goalSpeed = d_stabilizeSpeed;
	  }
  	else
  	{
  		goalAngle = (angleOfTorso * d_counterAngleWeight) + d_counterOffsetAngle;
  		goalSpeed = 9;
  	}
  	
  	if (abs(goalAngle) > abs(d_maxAngle))
  		goalAngle = d_maxAngle * (goalAngle/abs(goalAngle));
  	
  	goalAngle = (goalAngle/180.0) * M_PI;
  	goalSpeed = (goalSpeed / 180) * M_PI;

    con->addVar("Angle", goalAngle, goalAngle);     
    con->addVar("MaxSpeed", goalSpeed, goalSpeed); 
   // _debugLevel1("angle: " << angleOfTorso << ", speed: " << speedOfTorso);
  }
  else if(slot == Types::LLEG4 || slot == Types::RLEG4)
  {
    //float	goalSpeed = (3 / 180) * M_PI;
    con->addVar("Angle", 0, 0);
    con->addVar("MaxSpeed", 0, 0);
  }
  else 
  {
    con->addVar("Angle", 0, 0);
    con->addVar("MaxSpeed", 0, 0);
  }
  
  
  return goal;
}
