#include "steer.ih"

rf<Behavior::Goal> Steer::generateGoal(unsigned step, unsigned slot)
{
  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();
  
  if (step == 0)
  {
	  WorldModel& wm = SWorldModel::getInstance();
	  float ampMod = 1.;
	  float freqMod = 1.;
	  
	  // con->addVar("Var", 0, 1);
	  // Find position to go to
	  Vector3D pos;
	  
	  if (d_goal->findDeep("PosX"))
	    pos = d_goal->getVector("Pos");
	  else
	    pos = wm.getObject(Types::BALL).position.getMu();
	  
	  // Find amplitude modulation factor
	  if (d_goal->findDeep("AmplitudeMod"))
	  	ampMod = d_goal->getMean("AmplitudeMod");
	  
	  // Find Frequancy modulation factor
	  if (d_goal->findDeep("FrequencyMod"))
	  	freqMod = d_goal->getMean("FrequencyMod");
	  
	  // Angle to steer to
	  double angle = atan2(pos[0], pos[1]) / M_PI * 2; 
	  
	  _debugLevel1(pos << " angle: " << angle);
	  
	  // Determine factors for each joint
	  for (unsigned i = 0; i < Types::NJOINTS; ++i)
	  {
	    double f = 1.0;
	     
	    /*if (i >= Types::LLEG1 && i <= Types::LLEG6)
	      f = angle > 0 ? 1.0 : 1.0 + d_strength * angle;
	    else if (i >= Types::RLEG1 && i <= Types::RLEG6)
	      f = angle < 0 ? 1.0 : 1.0 - d_strength * angle;*/
	    
	    if (i >= Types::LLEG1 && i <= Types::LLEG6)
		    f = angle > 0 ? 1.0 : exp(-(angle * angle) * (d_strength * d_strength));
		  else if (i >= Types::RLEG1 && i <= Types::RLEG6)
		    f = angle < 0 ? 1.0 : exp(-(angle * angle) * (d_strength * d_strength));
	    
	    if (ampMod != 1. or freqMod != 1.)
	    {
				_debugLevel4("Close to goal, extra amplitude (" << ampMod << ") and frequency (" << freqMod << ") modulation.");    
	    	
	    	f *= ampMod;	 
	    }
	    
	    ostringstream jout;
	    jout << "AmpFactor" << i;
	    
	    con->addVar(jout.str(), f, f);
	    
	    if (freqMod != 1.)
		    con->addVar("FreqFactor", freqMod, freqMod);
	  }
  }

  return goal;
}
