#include "transition.ih"

#include <typeinfo>

void Transition::constructed()
{

	if (d_forid == ""){
		_debugLevel1("WARNING: variable d_forid is empty!");
		return;
	}
	_debugLevel1("generateGoal: " << d_forid); 
 
	rf<Behavior> ch = getBehavior(d_forid);
	
	if (!ch){
		_debugLevel1("WARNING: was unable to obtain rf<Behavior> from name specified in the XML for-node.");
		return;
	}

	d_for = dynamic_cast<PeriodicInterface *>(&*ch);
	
	if (!d_for){
		_debugLevel1("WARNING: was unable to obtain PeriodicInterface from Behavior.");
		return;
	}

	// Get the first step in the tree
	rf<AST::Node> first = d_tree->getChild(0);
	bool atleastone = false;

	// Iterate over all the slots in the first step
	for (size_t idx = 0; idx < first->size(); ++idx)
	{
		// Get the actual node
		rf<AST::Node> child = first->getChild(idx);
		
		// Test if the slot has behaviors
		if (!child->hasChildren())
			continue;

		rf<AST::Node> slot = child->getChild(0);
		
		if (slot->getType() != behaviorType)
			continue;
		
		try
		{
			rf<BehaviorNode> behavior = rf_cast<BehaviorNode>(slot);
			
			if (!behavior)
				continue;

			JointInterface &joint = dynamic_cast<JointInterface &>(*(behavior->getBehavior()));
					
			// Find the actual start position for this joint in d_for
			d_angles[idx] = Definition(joint.joint(), d_for->startPosition(joint.joint()));			
			atleastone = true;
		}
		catch (std::bad_cast &e)
		{
			_debugLevel2("Oops, cast didn't work out: " << e.what());
		}
	}
	
	d_capable = atleastone;
}

