#include "transition.ih"

rf<Behavior::Goal> Transition::generateGoal(unsigned step, unsigned slot)
{
	// The goal that needs to be satisfied for this behaviour, is a specific
	// angle, that's calculated in calcAngle. A MoveJointTo subbehaviour will
	// satisfy this goal
	
	double now = SWorldModel::getInstance().getTime();

	if (d_startTime == -1)
	{
		//WorldModel &wm = SWorldModel::getInstance();
		AgentModel& am = SAgentModel::getInstance();

		for (map<size_t, Definition>::iterator iter = d_angles.begin(); iter != d_angles.end(); ++iter)
			iter->second.from = am.getJoint(d_angles[slot].joint)->angle.getMu();

		d_startTime = now;
	}

	rf<Goal> goal = new Goal();
	rf<OrNode> dis = goal->addDisjunct();
	rf<AndNode> con = dis->addConjunct();

	_debugLevel1("Now: " << now << " d_capable: " << d_capable);

	if (d_startTime + d_period < now)
    {
		d_capable = false;
    }

	_debugLevel1("startTime: " << d_startTime << "    d_period: " << d_period << "   running?: " << not ((d_startTime + d_period) < now));
 
	if (d_angles.find(slot) == d_angles.end())
		return goal;

	double spend = (now - d_startTime);
	double factor = spend > 0 ? (spend / d_period) : 0;
	
	double to = d_angles[slot].to;
	double from = d_angles[slot].from;
	
	double angle = from + (to - from) * pow(factor, 3);

	con->addVar("Angle", angle, angle);

	return goal;
}

