#ifndef _BATS_TRANSITION_H_
#define _BATS_TRANSITION_H_

#include "behavior.hh"
#include "periodicinterface.hh"


namespace bats
{
	class Transition : public Behavior
	{
		struct Definition
		{
			Types::Joint joint;
			double from;
			double to;
			
			Definition() : from(0), to(0) {};
			Definition(Types::Joint j, double t) : joint(j), from(0), to(t) {};
		};
		
		PeriodicInterface *d_for;
		std::string d_forid;
		std::map<size_t, Definition> d_angles;
		
		double d_startTime;
		double d_period;
		bool d_capable;
		
		private:
			virtual rf<Goal> generateGoal(unsigned step, unsigned slot);
			virtual rf<State> getCurrentState();
			virtual ConfidenceInterval getCapability(rf<State> s, rf<Goal> g);
		protected:
			virtual void constructed();
		public:
			Transition(std::string const &id, std::string const &playerClass);

			void reset();
			virtual void update();
	};
};

#endif /* _BATS_TRANSITION_H_ */

