#include "trippleCombined.ih"

rf<Behavior::Goal> TrippleCombined::generateGoal(unsigned step, unsigned slot)
{
  WorldModel& wm = SWorldModel::getInstance();

  // Update all factors
  if (s_lastUpdate < wm.getTime())
  {
    for (unsigned i = 0; i < s_TrippleCombined.size(); ++i)
      s_TrippleCombined[i]->updateFactor(wm.getTime() - s_lastUpdate, s_TrippleCombined[i] == this);
    s_lastUpdate = wm.getTime();
  }
  
  double angle = 0;
  double fSum = 0;
  for (unsigned i = 0; i < s_TrippleCombined.size(); ++i)
  {
    vector<Params> params = s_TrippleCombined[i]->getParams()[slot];
    double f = s_TrippleCombined[i]->getFactor();
    if (f > 0)
    {
      _debugLevel1("Running trippleCombined : " << s_TrippleCombined[i]->ident() << " " << wm.getTime() << " " << f);
    }
    angle += f * calcAngle(params, s_TrippleCombined[i]->getStartTime());
    fSum += f;
  }
  
  if(fSum != d_factor)
  {
    _debugLevel1("Merging trippleCombined!");
  }
  
  if (fSum >1)
    angle /= fSum;
  
  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();

  _debugLevel4("Angle: " << angle);
  con->addVar("Angle", angle, angle);

  s_lastTrippleCombinedTime = wm.getTime();
  
  return goal;
}
