#include "walktoavoid.ih"
#include "math.hh"

rf<Behavior::Goal> WalkToAvoid::generateGoal(unsigned step, unsigned slot)
{
  WorldModel &wm = SWorldModel::getInstance();

  rf<Goal> goal = new Goal();
  rf<OrNode> dis = goal->addDisjunct();
  rf<AndNode> con = dis->addConjunct();

  Vector3D pos;
  if (d_goal->findDeep("PosX"))
     pos = d_goal->getVector("Pos");
  else
     pos = wm.getObject(Types::BALL).position.getMu();

   Vector3D dir;
  if (d_goal->findDeep("DirX"))
     dir = d_goal->getVector("Dir");
  else
     dir = ((wm.getObject(Types::GOAL1THEM).position.getMu() +
            wm.getObject(Types::GOAL2THEM).position.getMu()) / 2.0
             - wm.getObject(Types::BALL).position.getMu()).normalize();
  con->addVar("Dir", dir);

  pos = Avoider::correctIfCollides(pos, dir);

  if (differsSignificantly(pos, d_current_goal))
    d_current_goal = pos;

  con->addVar("Pos", d_current_goal);

  // if you are using goodpositionangle you can ignore the FinalPos thingy
  Vector3D finalGoal = d_current_goal;
  if (d_goal->findDeep("FinalPosX"))
  {
    finalGoal = d_goal->getVector("FinalPos");
  }
  con->addVar("FinalPos", finalGoal);

  float angle = atan2(d_current_goal[0], d_current_goal[1]) / M_PI * 2;

  if (d_goal->findDeep("Angle"))
      con->addVar("Angle", d_goal->getMean("Angle"),  d_goal->getMean("Angle"));

  float amplitudeMod = 1.;

  if (finalGoal)
  {
	float brakeDist = 1.0, targetSpeed = 0;
	if (d_goal->findDeep("BrakeDistance"))
	  brakeDist = d_goal->getMean("BrakeDistance");

	if (d_goal->findDeep("TargetSpeed"))
      targetSpeed = d_goal->getMean("TargetSpeed");

    if (finalGoal.length() < brakeDist)
	{
	  //moet nog e-macht functie gebruikt worden ipv lineaire downscaling
      if (d_goal->findDeep("SlowOnAngle"))
        amplitudeMod = Math::sigmoid(1 - (min(angle, 65.0f) / 65.0f), 1., targetSpeed, d_goal->getMean("SlowOnAngle"));
	  else
	    amplitudeMod = Math::sigmoid(1 - finalGoal.length() / brakeDist, 1., targetSpeed);
    }
  }
  con->addVar("AmplitudeMod", amplitudeMod, amplitudeMod);

  return goal;
}

