#include "behavior.ih"

#define _elseif(X) else if (b.getProp("type") == #X) \
    behavior = new X(b.getProp("id"), playerClass)

#define _elseif1(X) else if (b.getProp("type") == #X) \
    behavior = new X(b.getProp("id"))



rf<Behavior> Behavior::buildTree(XMLNode& b, string playerClass)
{
  Conf& conf = SConf::getInstance();

  // Make behavior of this node's type
  rf<Behavior> behavior;

  // --------------------------- Add new behavior types here -----------------------
  if (b.getProp("type") == "MoveHingeJoint")
    behavior = new MoveHingeJoint(b.getProp("id"), playerClass);

  _elseif(MoveUniversalJoint);
  _elseif(MoveJoint);
  _elseif(MoveJointTo);
  _elseif(MoveJointRubTo);
  _elseif(Wait);
  _elseif1(Beam);
  _elseif(ScriptPlayer);
  _elseif(Sine);
  _elseif(Empty);
  _elseif(WalkTo);
  _elseif(WalkToAvoid);
  _elseif(CanKickLeft);
  _elseif(CanKickRight);
  _elseif(CanTurnLeft);
  _elseif(CanTurnRight);
  _elseif(MiniWalkForward);
  _elseif(MiniWalkBackward);
  _elseif(Console);
  _elseif(GoodPositionAngle);
  _elseif(GoodPosition);
  _elseif(GetUpFromBelly);
  _elseif(GetUpFromBack);
  _elseif(BeamTo);
  _elseif(GetFromSide);
  _elseif(WalkStraightTo);
  _elseif(TurnTo);
  _elseif(KickTo);
  _elseif(Score);
  _elseif(StrafeLeft);
  _elseif(Challenge1);
  _elseif(StrafeRight);
  _elseif(StrafeToBall);
  _elseif(KeeperGetUp);
  _elseif(KeeperFall);
  _elseif(GuardGoal);
//  _elseif(RemoteControl);
  _elseif(KickBall);
  _elseif(PositionFoot);
  _elseif(PositionArm);
  _elseif(CatchFallBack);
  _elseif(CatchFallFront);
  _elseif(PushUpFromBelly);
  _elseif(PushUpFromBack);
  _elseif(Stabilize);
  _elseif(FallOver);
  _elseif(Reset);
  _elseif(GetUpQuick);
  _elseif(GetUpSlow);
  _elseif(GetUpSlowFromBelly);
  _elseif(GetUpSlowFromBack);

  _elseif(WaitForOpponent);
  _elseif(ZeroGoodPositionAngle);
  _elseif(CatchFallSide);
  _elseif(GoalKick);
  _elseif(KickOff);

  _elseif(Pass);
  _elseif(Position);
  _elseif(PlayMode);
  _elseif(FallCommit);
  _elseif(CanCurl);

  _elseif(Trainer);
  _elseif(KickTrainer);

  _elseif(MiniWalkCombined);
  _elseif(Defend);
  _elseif(Attack);

  _elseif(ScorpiusBlocker);

  _elseif(CoupledWalk);

  _elseif(CornerKick);
  _elseif(StayDown);

  _elseif(GAWalk);
  _elseif(QuitRun);
  _elseif(CubicSpline);
  _elseif(Transition);

  _elseif(MoveCOMTo);
  _elseif(SwingLeg);

  _elseif(ScalableSine);
  _elseif(Steer);

  _elseif(WalkAroundGoal);
  _elseif(Tactic);
  _elseif(BallInRange);

  _elseif(Dribble);
  _elseif(KickIn);


	_elseif(WalkToAngle);

	_elseif(WaitForKickOff);

	_elseif(FastKickBall);
	_elseif(PanicKick);

	_elseif(LocalBehavior);

  // --------------------------- End add new behavior types ------------------------

  if (!behavior)
    throw runtime_error(b.getProp("type") + ": unknown behavior type (trying to create behavior with id: " + b.getProp("id") + ")");

  behavior->setPlayerClass(playerClass);

  // Go through the slots
  for(XMLNode s = b.getChildren(); s; s = s.getNext())
  {
    if (s.getName() != "slot")
      continue;

    string index = s.getProp("index");
    unsigned step = atoi(index.substr(0, index.find("-")).c_str());
    unsigned slot = atoi(index.substr(index.find("-") + 1).c_str());

    _debugLevel2("Adding to slot " << step << "-" << slot << ":");

    for (XMLNode sb = s.getChildren(); sb; sb = sb.getNext())
      if (sb.getName() == "behavior")
      {
        _debugLevel2("\t" << sb.getProp("refid"));

        string path = "/conf/player-class[@id='" + playerClass + "']/behaviors/behavior[@id='" + sb.getProp("refid") + "']";
        try{
          string commit = sb.getProp("commit");
          string scicc = sb.getProp("scicc");

          XMLNodeSet set = conf.selectXPath(path);
          if (set.size() > 1)
          {
            //_foreach(s, set)
            //  cout << (*s).getContent() << endl;
            throw new BatsException(string() + "buildTree: '" + path + "' found multiple times.");
          }
          if (set.empty())
            throw new BatsException(string() + "buildTree: '" + path + "' not found.");

          XMLNode sb = set.front();
          rf<Behavior> subBehavior = buildTree(sb, playerClass);
          subBehavior->shouldCommit(commit == "1");
          subBehavior->shouldCommitIfChildrenCommitted(scicc == "1");
          behavior->addToSlot(subBehavior, step, slot);

        }
        catch(BatsException *e)
        {
            _debugLevel1("Problem with refid: " << sb.getProp("refid") << endl << ", xpath:" << path << ", what: " << e->what() << endl);
            _log(string("") + "Problem with refid: " + sb.getProp("refid") + string("\n,  xpath:") + e->what() + string("\n"));
            throw e;
        }
      }
  }
  //Check if we had already loaded the behavior
  /*
  if(s_behaviors.count(behavior->getId()) > 0)
  {
    cerr << "Double ID, behavior: " << behavior->getName() << "(" << behavior->getId() << ")" << " already in tree" << endl;
    //assert(false);
  }
  */
  s_behaviors[behavior->getId()] = behavior;

  return behavior;
}

