#ifndef _BATS_COCHLEA_HH_
#define _BATS_COCHLEA_HH_

#include "rf.hh"
#include "vector4.hh"
#include "../../SocketComm/AgentSocketComm/agentsocketcomm.hh"
#include "singleton.hh"
#include <vector>
#include "hashclasses.hh"
#include "playerinfo.hh"

namespace bats
{
  class Cochlea
  {
    friend class Singleton<Cochlea>;
    
  public:
    enum InfoID {

      iNone,
      
      iGameState,
      iUnum,
      iTeam,
      iGameTime,
      iPlayMode,
      
      iTime,         
      iNow,          ///< The current playing time.
      iStep,         ///< The time step (dt?)
      iVision,
      iVisionFlag1L, ///< First flag on the left side.
      iVisionFlag2L, ///< Second flag on the left side.
      iVisionFlag1R, ///< First flag on the right side.
      iVisionFlag2R, ///< Second flag on the right side.
      iVisionGoal1L, ///< First
      iVisionGoal2L, ///< 
      iVisionGoal1R, ///< 
      iVisionGoal2R, ///< 
      iVisionPlayer1,
      iVisionPlayer2,
      iVisionPlayer3,
      iVisionPlayer4,
      iVisionPlayer5,
      iVisionPlayer6,
      iVisionPlayer7,
      iVisionPlayer8,
      iVisionPlayer9,
      iVisionPlayerNone,
      iVisionOpponent1,
      iVisionOpponent2,
      iVisionOpponent3,
      iVisionOpponent4,
      iVisionOpponent5,
      iVisionOpponent6,
      iVisionOpponent7,
      iVisionOpponent8,
      iVisionOpponent9,
      iVisionOpponentNone,
      iVisionBall,   ///< Position of the ball.
      iUJ,
      iHJ,
      iName,
      iAxis,
      iRate,
      iAxis1,
      iRate1,
      iAxis2,
      iRate2,

      iHeadJoint1,
      iHeadJoint2,
      
      iLLegJoint1,
      iLLegJoint2,
      iLLegJoint3,
      iLLegJoint4,
      iLLegJoint5,
      iLLegJoint6,

      iRLegJoint1,
      iRLegJoint2,
      iRLegJoint3,
      iRLegJoint4,
      iRLegJoint5,
      iRLegJoint6,

      iLArmJoint1,  ///< 
      iLArmJoint2,  ///< 
      iLArmJoint3,
      iLArmJoint4,

      iRArmJoint1,
      iRArmJoint2,
      iRArmJoint3,
      iRArmJoint4,

      iGyro,
      iTorsoGyro,
      iTouch,
      iFootLeft,
      iFootLeft2,  /// dummy
      iFootRight,
      iFootRight2,
      iVal,
      
      iForceResistancePerceptor,
      iContact,
      iForce,

      iHear,
      
      iP, /// Player vision info.

      // Should always have the highest numeric value.
      iInfoID,
      
    };

    struct HearMessage
    {
      double time;
      double angle;
      std::string message;
      
      HearMessage(double t, double a, std::string const& m)
      : time(t), angle(a), message(m)
      {}
    };
  private:
    FBB::StringHash<InfoID> d_infoMap;
    FBB::StringHash<Types::PlayMode> d_playModeMap;

    Vector4F d_info[iInfoID];
    std::vector<HearMessage> d_hearMessages;

    std::string d_teamName;
    
    std::map<unsigned,PlayerInfo> d_players;
    std::map<unsigned,PlayerInfo> d_opponents;

    Cochlea();
    
    Vector4F parseVect(rf<Predicate> const &pred);

    Vector4F parseScalar(rf<Predicate> const &pred);

    void integrate(rf<Predicate> const &pred);

    PlayerInfo &getPlayerFromPid(unsigned pid);
    PlayerInfo &getOpponentFromPid(unsigned pid);
    
  public:
    Types::PlayMode getPlayMode(std::string const& mode) { return d_playModeMap[mode];}
    
    void setTeamName(std::string const& teamName) { d_teamName = teamName; }
    
    void setTranslation(std::string const& from, std::string const& to) { d_infoMap[from] = d_infoMap[to]; }
    InfoID translateInfo(std::string const &name) { return d_infoMap[name]; }

    void update();

    Vector4F getInfo(InfoID id) { return d_info[id]; }

    //    PlayerInfo &getPlayer(unsigned unum); // DO NOT USE.
    //    PlayerInfo &getOpponent(unsigned unum); // DO NOT USE.

    typedef std::map<unsigned,PlayerInfo>::const_iterator player_iterator;

    player_iterator beginPlayers() const;
    player_iterator endPlayers() const;

    player_iterator beginOpponents() const;
    player_iterator endOpponents() const;

  };
  
  typedef Singleton<Cochlea> SCochlea;

  inline Cochlea::player_iterator Cochlea::beginPlayers() const
  {
    return d_players.begin();
  }

  inline Cochlea::player_iterator Cochlea::endPlayers() const
  {
    return d_players.end();
  }

  inline Cochlea::player_iterator Cochlea::beginOpponents() const
  {
    return d_opponents.begin();
  }

  inline Cochlea::player_iterator Cochlea::endOpponents() const
  {
    return d_opponents.end();
  }  

}

#endif

