/*
 *  Little Green BATS (2007), AI department, University of Groningen
 *
 *  Authors: 	Martin Klomp (martin@ai.rug.nl)
 *		Mart van de Sanden (vdsanden@ai.rug.nl)
 *		Sander van Dijk (sgdijk@ai.rug.nl)
 *		A. Bram Neijt (bneijt@gmail.com)
 *		Matthijs Platje (mplatje@gmail.com)
 *
 *	All students of AI at the University of Groningen
 *  at the time of writing. See: http://www.ai.rug.nl/
 *
 *  Date: 	July 27, 2007
 *
 *  Website:	http://www.littlegreenbats.nl
 *
 *  Comment:	Please feel free to contact us if you have any 
 *		problems or questions about the code.
 *
 *
 *  License: 	This program is free software; you can redistribute 
 *		it and/or modify it under the terms of the GNU General
 *		Public License as published by the Free Software 
 *		Foundation; either version 3 of the License, or (at 
 *		your option) any later version.
 *
 *   		This program is distributed in the hope that it will
 *		be useful, but WITHOUT ANY WARRANTY; without even the
 *		implied warranty of MERCHANTABILITY or FITNESS FOR A
 *		PARTICULAR PURPOSE.  See the GNU General Public
 *		License for more details.
 *
 *   		You should have received a copy of the GNU General
 *		Public License along with this program; if not, write
 *		to the Free Software Foundation, Inc., 59 Temple Place - 
 *		Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef __INC_BATS_FITNESSWALKSTRAIGHT2_HH__
#define __INC_BATS_FITNESSWALKSTRAIGHT2_HH__

#include <iostream>
#include "cochlea.hh"
#include "fitnessfunction.hh"
#include "worldmodel.hh"
#include "math.hh"
#include <cmath>
#include <limits>
#include "vector4.hh"
#include "vector3.hh"
#include <vector>
#include "logger.hh"

namespace bats {

  /**
   *  This fitness function is used to learn to walk.
   */
  class FitnessWalkStraight2 : public FitnessFunction {

    double d_fitness;

    FitnessWalkStraight2(FitnessWalkStraight2 const &other); // NI
    FitnessWalkStraight2 &operator=(FitnessWalkStraight2 const &other); // NI

  public:

    FitnessWalkStraight2()
    :
      d_fitness(0)
    {}

    virtual ~FitnessWalkStraight2() {}

    virtual void initCommand(AgentSocketComm &comm)
    {
      //      std::cerr << "Beaming to (-500,0,0)" << std::endl;
      comm.beam(Vector3D(0.0,-500.0,0.0));

    }

    virtual void update()
    {
      //d_fitness = SCochlea::getInstance().getInfo(Cochlea::iVisionBall)[0];
	  Cochlea& cochlea = SCochlea::getInstance();
  
	  Vector3D worldRight = Math::polarToCartesian(cochlea.getInfo(Cochlea::iVisionFlag1R)) - Math::polarToCartesian(cochlea.getInfo(Cochlea::iVisionFlag1L));
	  Vector3D worldForward = Math::polarToCartesian(cochlea.getInfo(Cochlea::iVisionFlag1L)) - Math::polarToCartesian	(cochlea.getInfo(Cochlea::iVisionFlag2L));
	  Vector3D worldUp = worldRight.crossProduct(worldForward);

	  Matrix4D worldTransform(worldRight.normalize(), worldForward.normalize(), worldUp.normalize());
	  Matrix4D invWorldTransform = worldTransform.inverse();
  
	  Vector3F ballPos = invWorldTransform * Math::polarToCartesian(cochlea.getInfo(Cochlea::iVisionBall));
	  d_fitness = fabs(ballPos.getX()) + fabs(ballPos.getY()*1.5);
	  
    }

    virtual void print() const
    {
		std::cout << d_fitness << std::endl; // << "\t" << SWorldModel::getInstance().getFieldNormal() << std::endl;	
    }


  };

};

#endif // __INC_BATS_FITNESSWALKSTRAIGHT2_HH__
