#ifndef _INCLUDED_LINEARFUNCTION_
#define _INCLUDED_LINEARFUNCTION_

#include "../function.hh"
#include <cmath>

inline double mar_abs(double in)
{
    return in > 0.0 ? in : -in;
}

namespace bats
{
class LinearFunction : public Function<double, double>
{
    double d_angle, d_length, d_factor;

    public:
        LinearFunction(double angle, double length, double factor)
            :
            d_angle(angle), d_length(length), d_factor(factor)
        {}


        double operator()(double const &in)
        {
            double value = mar_abs(in - d_angle);
            if (value > 180.0)
               value = -(360. - value);
            value = value / (d_length / 2.0) - 1.0;
            value = -value > 0.0 ? -value : 0.0;
            return value;
        }

    private:
};

}        
#endif
