#ifndef _INCLUDED_POSTURE_
#define _INCLUDED_POSTURE_

#include <iostream>
#include <vector>
#include <map>
#include "../../Types/types.hh"
#include "../../Behavior/behavior.hh"
#include "rf.hh"
#include "refable.hh"
#include "predicate.hh"

class Posture
{
  friend std::ostream &operator<<(std::ostream &out, Posture const &in);

public:
  typedef bats::Types::Joint Joint_;
  typedef std::map<Joint_, double> PosturePosition;

private:
  std::vector<Joint_> d_ids;
  std::vector<double> d_joint_positions;
  
 public:
  Posture(){}
  Posture(PosturePosition const &posture_position);

  void addJoint(Joint_ joint, double pos);
  bats::rf<bats::Behavior::Goal> createGoalNode(size_t slot);

  PosturePosition getPosturePosition();

  Posture operator*(double other);
 private:
};

inline std::ostream &operator<<(std::ostream &out, Posture const &in)
{
  out << "posture: ";
  for (size_t i(0); i < in.d_joint_positions.size(); ++i)
    {
      out << static_cast<signed int>(in.d_ids[i]) << ": " << in.d_joint_positions[i] << std::endl;
    }
  return out;
}

#endif
