#include "posturecombiner.ih"

Posture PostureCombiner::operator()(double const &in)
{
  vector<double> counter(Types::NJOINTS);
  fill(counter.begin(), counter.end(), 0);

  vector<double> joint_value(Types::NJOINTS);
  fill(joint_value.begin(), joint_value.end(), 0);
  
  for (size_t i(0); i < d_functions.size(); ++i)
    {
      Posture posture = (*d_functions[i])(in);
      Posture::PosturePosition posturePosition = posture.getPosturePosition();
      
      Posture::PosturePosition::iterator it = posturePosition.begin();
      Posture::PosturePosition::iterator end = posturePosition.end();

      for (;it != end; ++it)
	{
	  counter[(*it).first] += d_weights[i];
	  joint_value[(*it).first] += d_weights[i] * (*it).second;
	}
    }

  for (size_t i(0); i < joint_value.size(); ++i)
    if (counter[i] != 0.0)
      joint_value[i] /= counter[i];


  Posture::PosturePosition ret;
  for (size_t i(0); i < joint_value.size(); ++i)
	ret[static_cast<Posture::Joint_>(i)] = joint_value[i];

  return Posture(ret);
}
