#include "vectorpredictor.ih"

VectorPredictor::GuessResult VectorPredictor::predictClosest(Vector3D const &reference, double from, double to, size_t n_steps, size_t depth) const
{
  GuessResult none = {Vector3D(0, 0, 0), 9999999, 0};
  if (d_times_updated < d_vectors.size() + 3)
    return none;

  double step_size = (to - from) / n_steps;
  Vector3D guess, best_guess;
  double guess_time(from);
  double min_distance(999999999);

  for (double time(from); time < to; time += step_size)
  {
    predict(time, &guess);
    double distance(0);
    if ((distance = (guess - reference).length()) < min_distance)
    {
      min_distance = distance, guess_time = time, best_guess = guess;
    }
  }

  if (depth <= 0)
  {
    GuessResult result = {guess, min_distance, guess_time};
    return result;
  }
  else
  {
    double from = guess_time - step_size;
    double to = guess_time + step_size;
    GuessResult result = predictClosest(reference, from, to, n_steps, depth - 1);
    return result;
  }
}
