#include "vectorpredictor.ih"

Vector3D VectorPredictor::convertToGlobalVector(Vector3D const &in)
{
  Cochlea& cochlea = SCochlea::getInstance();

  Vector3D worldRight = Math::polarToCartesian(cochlea.getInfo(Cochlea::iVisionFlag1R)) - Math::polarToCartesian(cochlea.getInfo(Cochlea::iVisionFlag1L));
  Vector3D worldForward = Math::polarToCartesian(cochlea.getInfo(Cochlea::iVisionFlag1L)) - Math::polarToCartesian(cochlea.getInfo(Cochlea::iVisionFlag2L));
  Vector3D worldUp = worldRight.crossProduct(worldForward);

  Matrix4D worldTransform(worldRight.normalize(), worldForward.normalize(), worldUp.normalize());
  Matrix4D invWorldTransform = worldTransform.inverse();

  Vector3D reference = invWorldTransform * Math::polarToCartesian(cochlea.getInfo(Cochlea::iVisionFlag1L));
  //return invWorldTransform * Math::polarToCartesian(in) - reference;
  return invWorldTransform * in - reference;
}

void VectorPredictor::update(Vector3D const &new_input)
{
  Vector3D temp_vector = new_input;
  //cout << temp_vector.getX() << " " << temp_vector.getY() << " ";
  WorldModel &wm = SWorldModel::getInstance();

  double current_time = wm.getTime();
  double time_diff = current_time - d_last_time;
  d_last_time = current_time;
  
  vector<Vector3D> new_vectors(d_vectors);
  
  new_vectors[0] = temp_vector;
  vector<Vector3D>::const_iterator it = d_vectors.begin() + 1; 
  vector<Vector3D>::const_iterator end = d_vectors.end(); 

  size_t i(1);
  //cout << "---" << endl;
  //cout << "0 v: " << new_vectors[0] << endl;
  //cout << time_diff << endl;
  for (; it != end; ++it, ++i)
    {
      //cout << i << " v: " << (new_vectors[i] = (new_vectors[i - 1] - *(it - 1)) / time_diff) << endl;
      new_vectors[i] = (new_vectors[i - 1] - *(it - 1)) / time_diff;
    }

  d_vectors.swap(new_vectors);


  // leaky integrators updaten
  {
    vector<LeakyIntegrator<Vector3D> >::iterator li_it = d_li_vectors.begin();
    vector<LeakyIntegrator<Vector3D> >::iterator li_end = d_li_vectors.end(); 
    
    vector<Vector3D>::const_iterator it = d_vectors.begin(); 

    for (; li_it != li_end; ++li_it, ++it)
      *li_it = *it;
  }
  
  ++d_times_updated;
}
