#ifndef _INCLUDED_VECTORPREDICTOR_
#define _INCLUDED_VECTORPREDICTOR_

#include "../Vector3/vector3.hh"
#include <vector>

#include "../leakyintegrator/leakyintegrator.h"

namespace bats
{
class VectorPredictor
{
 public:
  struct GuessResult
  {
    Vector3D closest;
    double min_distance;
    double time;
  };

 private:
  double d_last_time;
  
  std::vector<Vector3D> d_vectors;
  std::vector<LeakyIntegrator<Vector3D> > d_li_vectors;
  
  size_t d_times_updated;

 public:
  VectorPredictor(size_t n = 1);

  void update(Vector3D const &new_input);
  GuessResult predictClosest(Vector3D const &reference, double max_time = 3.0) const;

  Vector3D predict(double time) const;
  void predict(double time_from_now, Vector3D *return_vec) const;

  Vector3D convertToGlobalVector(Vector3D const &new_input);

 private:
  GuessResult predictClosest(Vector3D const &reference, double from, double to, size_t n_steps, size_t depth) const;

  size_t faculty(size_t n) const;
};

}        
#endif
