#ifndef __INC_BATS_PLAYERINFO_HH__
#define __INC_BATS_PLAYERINFO_HH__

#include "vector3.hh"

#include "agentmodel.hh"

namespace bats {

  class PlayerInfo {

    static const unsigned c_histLength = 5;

    float d_lastSeen;
    float d_lastStep;

    unsigned d_id;

    Matrix4D d_transform;

    //    Vector3F d_positionHist[c_histLength];
    Vector3F d_velocityHist[c_histLength];

    unsigned d_histPos;

    Vector3F d_longPosIntergrate;

    Vector3F d_lastPosition; // the last seen position.

    Vector3F d_position;
    Vector3F d_worldPosition;
    Vector3F d_estVelocity;
    Vector3F d_estWorldVelocity;

    bool d_thinkGoally;

  public:

  
    void setID(unsigned id);

    unsigned getID() const;

    void updatePosition(Vector3F const &pos, float time);

    float getLastStep() const;

    Vector3F const &getPosition() const;

    Vector3F const &getWorldPosition() const;

    Vector3F const &getEstVelocity() const;

    Vector3F const &getEstWorldVelocity() const;

    bool thinkGoally() const;

    bool onGround() const;
    
  inline PlayerInfo()
    : d_id(0)
  {}
  
  inline ~PlayerInfo()
  {}

  };


  inline void PlayerInfo::setID(unsigned id)
  {
    d_id = id;
  }

  inline float PlayerInfo::getLastStep() const
  {
    return d_lastStep;
  }

  inline unsigned PlayerInfo::getID() const
  {
    return d_id;
  }

  inline Vector3F const &PlayerInfo::getPosition() const
  {
    return d_position;
  }

  inline Vector3F const &PlayerInfo::getWorldPosition() const
  {
    return d_worldPosition;
  }

  inline Vector3F const &PlayerInfo::getEstVelocity() const
  {
    return d_estVelocity;
  }

  inline Vector3F const &PlayerInfo::getEstWorldVelocity() const
  {
    return d_estWorldVelocity;
  }

  inline bool PlayerInfo::thinkGoally() const
  {
    return d_thinkGoally;
  }
  
  inline bool PlayerInfo::onGround() const
  {
    return d_worldPosition[2] < 0.2;
  }

};


#endif // __INC_BATS_PLAYERINFO_HH__
