#!/usr/bin/env python
import sys
import popen2
import signal
import subprocess
import pygtk
import os
pygtk.require('2.0')
import gtk
import gobject
import time

def loadConstraints(filename = 'constraints.txt'):
  #Return a dictionary of paired constraints
  f = file(filename, 'r')
  constraints = dict()
  for l in f.xreadlines():
    v = l.split()
    if len(v) > 1:
      constraints[v[0]] = [int(v[1]), int(v[2])]
  return constraints

class BatsConsoleGui:

    # This is a callback function. The data arguments are ignored
    # in this example. More on callbacks below.
    def hello(self, widget, data=None):
        print "Hello World", data

    def read(self):
      #Read input from proc.stdin
      r = ''
      try:
        r = self.proc.stdout.read()
      except:
        pass
      sys.stdout.write(r)
      self.output.get_buffer().insert(self.output.get_buffer().get_end_iter(), r)

    def delete_event(self, widget, event, data=None):
        # If you return FALSE in the "delete_event" signal handler,
        # GTK will emit the "destroy" signal. Returning TRUE means
        # you don't want the window to be destroyed.
        # This is useful for popping up 'are you sure you want to quit?'
        # type dialogs.
        print "TERMING", self.proc.pid
        os.kill(self.proc.pid, signal.SIGTERM)
        #print "Killing", self.proc.pid

        print "delete event occurred"

        # Change FALSE to TRUE and the main window will not be destroyed
        # with a "delete_event".
        return False

    def destroy(self, widget, data=None):
        print "destroy signal occurred"
        self.proc.stdin.close()
        self.proc.stdout.close()
        gtk.main_quit()

    def setJoint(self, adjustment, data):
      self.proc.stdin.write(data+" "+str(int(adjustment.value))+"\n")#data, adjustment.value
      self.read()

    def send(self, widget, data):
      self.proc.stdin.write(data+"\n")
      if data == 'reset':
        for adj in self.adjustments.values():
          adj.set_value(0)
      if data == 'script':
        time.sleep(0.1)
      self.read()

    def __init__(self, proc):
        # create a new window
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.resize(600,400)
        self.proc = proc
        # When the window is given the "delete_event" signal (this is given
        # by the window manager, usually by the "close" option, or on the
        # titlebar), we ask it to call the delete_event () function
        # as defined above. The data passed to the callback
        # function is NULL and is ignored in the callback function.
        self.window.connect("delete_event", self.delete_event)

        # Here we connect the "destroy" event to a signal handler.
        # This event occurs when we call gtk_widget_destroy() on the window,
        # or if we return FALSE in the "delete_event" callback.
        self.window.connect("destroy", self.destroy)

        self.output = gtk.TextView()

        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        sw.add(self.output)


        box = gtk.HBox()
        lbox = gtk.VBox()
        rbox = gtk.VBox()
        cbox = gtk.VBox()
        cbox.pack_start(sw)
        controlsbox = gtk.HBox()
        scriptButton = gtk.Button("Script")
        scriptButton.connect("clicked", self.send, "s")
        controlsbox.pack_start(scriptButton, expand=False)

        resetButton = gtk.Button("Reset")
        resetButton.connect("clicked", self.send, "reset")
        controlsbox.pack_start(resetButton, expand=False)


        cbox.pack_start(controlsbox, expand=False)

        box.pack_start(lbox)
        box.pack_start(cbox)
        box.pack_start(rbox)

        # Sets the border width of the window.
        self.window.set_border_width(10)
        self.scales = []
        self.joints = []
        for limb in [['arm', 4], ['leg' ,6]]:
          for i in xrange(1,limb[1]+1):
            self.joints.append('l'+limb[0]+str(i))
            self.joints.append('r'+limb[0]+str(i))
        self.adjustments = dict()
        constraints = loadConstraints()
        for joint in self.joints:

          adjustment = gtk.Adjustment(value=0, lower=constraints[joint][0], upper=constraints[joint][1], step_incr=1, page_incr=0, page_size=0)
          self.adjustments[joint] = adjustment
          adjustment.connect("value_changed", self.setJoint, joint)
          scale = gtk.HScale(adjustment)
          scale.set_digits(0)
          #self.window.add(scale)
          scale.set_update_policy(gtk.UPDATE_DELAYED)
          labelscalebox = gtk.HBox()
          labelscalebox.pack_start(gtk.Label(joint), expand=False)
          labelscalebox.pack_end(scale)
          if joint[0] == 'r':
            rbox.pack_start(labelscalebox)
          else:
            lbox.pack_start(labelscalebox)
          self.scales.append(scale)

        # Creates a new button with the label "Hello World".
        self.button = gtk.Button("Hello World")

        # When the button receives the "clicked" signal, it will call the
        # function hello() passing it None as its argument.  The hello()
        # function is defined above.
        self.button.connect("clicked", self.hello, "poeo")

        # This will cause the window to be destroyed by calling
        # gtk_widget_destroy(window) when "clicked".  Again, the destroy
        # signal could come from here, or the window manager.
        #self.button.connect_object("clicked", gtk.Widget.destroy, self.window)

        # This packs the button into the window (a GTK container).
        #self.window.add(self.button)
        self.window.add(box)
        # The final step is to display this newly created widget.
        #self.button.show()
        #box.show()
        # and the window
        self.window.show_all()

    def main(self):
        # All PyGTK applications must have a gtk.main(). Control ends here
        # and waits for an event to occur (like a key press or mouse event).
        gtk.main()


def unblock(fd):
  import fcntl
  import os
  flags = fcntl.fcntl(fd, fcntl.F_GETFL)
  fcntl.fcntl(fd, fcntl.F_SETFL, flags| os.O_NONBLOCK)


#Gui for the console, needs humanoind thing as first argument
if __name__ == "__main__":
  if len(sys.argv) < 2:
    print "Use something like: ./console.py './humanoidbat -c opstaan.xml -u 3'"
    exit(1)
  proc = subprocess.Popen(sys.argv[1].split(' '),
                        shell=False,
                        stdin=subprocess.PIPE,
                        stdout=subprocess.PIPE,
                        )
  unblock(proc.stdout.fileno())

  #(child_stdout, child_stdin) = popen2.popen2(sys.argv[1], bufsize=-1, mode)
  bcg = BatsConsoleGui(proc)
  bcg.main()

