/* GPL */

#include "humanoidbat.hh"
#include <iostream>
#include <cstdlib>
#include <stdexcept>
#include <signal.h>
#include "Logger/logger.hh"
#include "fitnesswalk.hh"
#include "fitnesswalkstraight.hh"
#include "fitnesswalkstraight2.hh"
#include "fitnessturn.hh"

#include "Tactiek/tactiek.h"

using namespace bats;
using namespace std;

void sigINT(int a)
{
	Logger::getInstance().destroy();
  cerr << "Caught sigint, exiting.." << endl;
  exit(1);
}

void sigTERM(int a)
{
	Logger::getInstance().destroy();
  cerr << "Caught sigterm, exiting.." << endl;
  exit(3);
}

void sigKILL(int a)
{
	Logger::getInstance().destroy();
  cerr << "Caught sigkill, exiting.." << endl;
  exit(2);
}

int main(int argc, char const ** argv)
try
{
  // TODO: We need a better way to exit! (better for the cluster)

  signal(SIGINT,sigINT);
  signal(SIGKILL,sigKILL);
  signal(SIGTERM,sigTERM);

  srand(time(0));

  cout << "This is the stdout, see others for more information." << endl;

  bool errorLogFile = false;
  unsigned errorBufferSize = 0;
  string teamName = "BATS";
  string conffile = "conf.xml";
  string host = "localhost";
  string fitness = "";
  unsigned unum = 0;
  unsigned port = 3100;
  
  for (int a = 1; a < argc; a++)
    if (string(argv[a]) == "-t")
      {assert(a+1 < argc); teamName = argv[++a];}
    else if (string(argv[a]) == "-c")
      {assert(a+1 < argc); conffile = argv[++a];}
    else if (string(argv[a]) == "-h")
      {assert(a+1 < argc); host = argv[++a];}
    else if (string(argv[a]) == "-l")
      errorLogFile = true;
    else if (string(argv[a]) == "-b")
      {assert(a+1 < argc); errorBufferSize = atoi(argv[++a]);}
    else if (string(argv[a]) == "-u")
      {assert(a+1 < argc); unum = atoi(argv[++a]);}
    else if (string(argv[a]) == "-p")
      {assert(a+1 < argc); port = atoi(argv[++a]);}
    else if (string(argv[a]) == "-f")
      {assert(a+1 < argc); fitness = argv[++a];}

  ostringstream logfile;
  logfile << "player" << teamName << unum << ".log";
  Logger::initialize(logfile.str());
  //  cerr << "Unum: " << unum << endl;

  HumanoidBat a(teamName, conffile, host, port, unum);

  if (!fitness.empty()) {
    if (fitness == "walk")
      a.setFitness(new FitnessWalk);
	else if (fitness == "walkstraight")
      a.setFitness(new FitnessWalkStraight);
	else if (fitness == "walkstraight2")
      a.setFitness(new FitnessWalkStraight2);
    else if (fitness == "turn")
      a.setFitness(new FitnessTurn);
  }

  a.run();

  return 0;
}
catch (BatsException *e)
{
  cerr << "Bats exception trace: " << endl << e->messages().str() << endl;
}
/*catch(exception e)
{
  //All is LOST... nothing to do here but die
  //Logger::getInstance().log(std::string() + "Caught std::exception (" + typeid(e).name() + "): " + e.what());
  //Logger::destroy();
  cerr << "Caught std::exception (" << typeid(e).name() << "): " << e.what() << endl;
}*/

