#include "shootfield.ih"

void ShootField::calculate(Check check)
{
    WorldModel &wm = SWorldModel::getInstance();
 
    clear();

    double angle_diff = 2 * M_PI / d_res;
    double angle = 0;

    //playerzz
    if (check & PLAYERS)
    {
        vector<double>::iterator it = d_values.begin();
        vector<double>::iterator end = d_values.end();

        WorldModel::player_iterator it_pl = wm.beginPlayers();
        WorldModel::player_iterator end_pl = wm.endPlayers();

        for (; it_pl != end_pl; ++it_pl)
        {   
            Vector3D pos = it_pl->second.getPosition();
            double playerAngle = atan2(pos.getY(), pos.getX());
            double playerDistance = pos.length();  
 
            NormDist dis(playerAngle, 0.3, 0.5);
            angle = -M_PI;
            for (; it != end; ++it, angle += angle_diff)
            {
                *it += dis(angle);
            }
        }
    }

    //opponents
    if (check & OPPONENTS)
    {
        vector<double>::iterator it = d_values.begin();
        vector<double>::iterator end = d_values.end();

        WorldModel::player_iterator it_op = wm.beginOpponents();
        WorldModel::player_iterator end_op = wm.endOpponents();

        for (; it_op != end_op; ++it_op)
        {
            Vector3D pos = it_op->second.getPosition();

            double opponentAngle = atan2(pos.getY(), pos.getX());
            double opponentDistance = pos.length();  

            NormDist dis(opponentAngle, 0.3, -1);
            angle = -M_PI;
            for (; it != end; ++it, angle += angle_diff)
            {
                *it += dis(angle);
            }
        }
    }

    //das goal
    if (check & GOAL)
    {
        Vector3D pos = (wm.getObject(Types::GOAL1THEM).position.getMu() + wm.getObject(Types::GOAL2THEM).position.getMu()) / 2.0;
        double goalAngle = atan2(pos.getY(), pos.getX());
        double goalDistance = pos.length(); 

        vector<double>::iterator it = d_values.begin();
        vector<double>::iterator end = d_values.end();

        double angle = -M_PI;

        LinearFunction dis(goalAngle, 1, 3);
        
        for (; it != end; ++it, angle += angle_diff)
            *it += dis(angle);
    }

    //global forward
    if (check & FORWARD)
    {     
        vector<double>::iterator it = d_values.begin();
        vector<double>::iterator end = d_values.end();

        Vector3D forward_dir = wm.getObject(Types::FLAG1THEM).position.getMu() - 
                               wm.getObject(Types::FLAG1US).position.getMu();
        forward_dir = forward_dir.normalize();

        double forwardAngle = atan2(forward_dir.getY(), forward_dir.getX());
        double angle = -M_PI;

        LinearFunction dis(forwardAngle, 1, 0.5);
        for (; it != end; ++it, angle += angle_diff)
            *it += dis(angle);
    }

    {
        double min_val = *min_element(d_values.begin(), d_values.end());
        double max_val = *max_element(d_values.begin(), d_values.end());
        
        d_highest_value = max_val;
        double highest_index = (max_element(d_values.begin(), d_values.end()) - d_values.begin());
        d_highest_angle = -M_PI + (angle_diff * highest_index);


        //teken / debug spul
        if ((max_val - min_val) == 0.0)
            max_val += 1;
  
        vector<double>::iterator it = d_values.begin();
        vector<double>::iterator end = d_values.end();

        //cout << "\rm: " << max_val << "," << min_val << " :";
        //for (; it != end; ++it)
        //   cout << ((size_t) (9 * (*it - min_val) / (max_val - min_val)));

    }

}

