#!/usr/bin/perl

use IO::Socket;
use Math::Trig;

sub prepareMsg
{
  my $msg = shift;
  my $len = length($msg);
  my $pref = pack 'N',$len;
  return $pref.$msg;
}

my $host = shift || 'localhost';
my $port = shift || 3200;
my $proto = getprotobyname("tcp");

my $iaddr = inet_aton($host);
my $paddr = sockaddr_in($port, $iaddr);

print "Target (0=goal, 1=player): ";
my $target = <>;

$target == 0 or $target == 1 or die "Invalid target!\n";

print "Give minimum x distance from target (meters): ";
my $minDist = <>;

print "Give maximum x distance from target (meters): ";
my $maxDist = <>;

if ($target == 0) {
  print "Give maximum y distance from center (meters): ";
  $maxSide = <>;
}

print "Give maximum latitude (degrees): ";
my $maxLat = <>;

print "Give minimum velocity (m/s): ";
my $minVel = <>;

print "Give maximum velocity (m/s): ";
my $maxVel = <>;

print "Give time between shots (s): ";
my $period = <>;

$/ = "\0";

print "Connecting to server $host:$port...\n";

my $sock = new IO::Socket::INET ( PeerAddr => $host, PeerPort => $port, Proto => 'tcp',  );
die "Could not create socket: $!\n" unless $sock;

print "Done!\n";

my $fieldLength = 0;
my $fieldWidth = 0;
my $goalWidth = 0;

while ($fieldLength == 0) {
  $line = <$sock>;
  if ($line =~ /FieldLength ([\d\.]+).*?FieldWidth ([\d\.]+).*?GoalWidth ([\d\.]+)/) {
    $fieldLength = $1;
    $fieldWidth = $2;
    $goalWidth = $3;
  }
}

print "Field parameters: $fieldLength $fieldWidth $goalWidth\n";

my $lastShot = -1;
print $sock prepareMsg("(ball (pos 0 0 0.3)(vel 0 0 0))");
while (<$sock>) {
  #print;
  if ($lastShot < 0 || time - $lastShot > $period) {
    my $x = ($fieldLength / 2.0) - (rand($maxDist - $minDist) + $minDist);
    my $y = rand(2 * $maxSide) - $maxSide;
    my $z = 0.3;
    my $lat = rand($maxLat);
    my $vel = rand($maxVel - $minVel) + $minVel;

    my $goalX = ($fieldLength / 2);
    my $goalY = rand($goalWidth) - $goalWidth / 2.0;

    my $dx = $goalX - $x;
    my $dy = $goalY - $y;
    my $dist = sqrt($dx * $dx + $dy * $dy);
    my $dz = $dist * tan($lat / 180.0 * pi);

    my $vecLen = sqrt($dx * $dx + $dy * $dy + $dz * $dz);
    $dx *= $vel / $vecLen;
    $dy *= $vel / $vecLen;
    $dz *= $vel / $vecLen;

    my $msg = "(ball (pos $x $y 0.3)(vel $dx $dy $dz))";
    print $sock prepareMsg($msg);
    print "$msg\n";

    $lastShot = time;
  }
}



close($sock) or die "Failed closing socket: $!\n";

